# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional, Union

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.abtesting.models.currency import Currency
from algoliasearch.abtesting.models.filter_effects import FilterEffects

_ALIASES = {
    "add_to_cart_count": "addToCartCount",
    "add_to_cart_rate": "addToCartRate",
    "average_click_position": "averageClickPosition",
    "click_count": "clickCount",
    "click_through_rate": "clickThroughRate",
    "conversion_count": "conversionCount",
    "conversion_rate": "conversionRate",
    "currencies": "currencies",
    "description": "description",
    "estimated_sample_size": "estimatedSampleSize",
    "filter_effects": "filterEffects",
    "index": "index",
    "no_result_count": "noResultCount",
    "purchase_count": "purchaseCount",
    "purchase_rate": "purchaseRate",
    "search_count": "searchCount",
    "tracked_search_count": "trackedSearchCount",
    "traffic_percentage": "trafficPercentage",
    "user_count": "userCount",
    "tracked_user_count": "trackedUserCount",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Variant(BaseModel):
    """
    Variant
    """

    add_to_cart_count: int
    """ Number of add-to-cart events for this variant. """
    add_to_cart_rate: Optional[float] = None
    """ [Add-to-cart rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#add-to-cart-rate) for this variant.  """
    average_click_position: Optional[int] = None
    """ [Average click position](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#click-position) for this variant.  """
    click_count: int
    """ Number of click events for this variant. """
    click_through_rate: Optional[float] = None
    """ [Click-through rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#click-through-rate) for this variant.  """
    conversion_count: int
    """ Number of click events for this variant. """
    conversion_rate: Optional[float] = None
    """ [Conversion rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#conversion-rate) for this variant.  """
    currencies: Optional[Dict[str, Currency]] = None
    """ A/B test currencies. """
    description: Optional[str] = None
    """ Description for this variant. """
    estimated_sample_size: Optional[int] = None
    """ Estimated number of searches required to achieve the desired statistical significance.  The A/B test configuration must include a `mininmumDetectableEffect` setting for this number to be included in the response.  """
    filter_effects: Optional[FilterEffects] = None
    index: str
    """ Index name of the A/B test variant (case-sensitive). """
    no_result_count: Union[int, None]
    """ Number of [searches without results](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#searches-without-results) for this variant. """
    purchase_count: int
    """ Number of purchase events for this variant. """
    purchase_rate: Optional[float] = None
    """ [Purchase rate](https://www.algolia.com/doc/guides/search-analytics/concepts/metrics/#purchase-rate) for this variant.  """
    search_count: Union[int, None]
    """ Number of searches for this variant. """
    tracked_search_count: Optional[int] = None
    """ Number of tracked searches. Tracked searches are search requests where the `clickAnalytics` parameter is true. """
    traffic_percentage: int
    """ Percentage of search requests each variant receives. """
    user_count: Union[int, None]
    """ Number of users that made searches to this variant. """
    tracked_user_count: Union[int, None]
    """ Number of users that made tracked searches to this variant. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Variant from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Variant from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["currencies"] = (
            dict((_k, Currency.from_dict(_v)) for _k, _v in obj["currencies"].items())
            if obj.get("currencies") is not None
            else None
        )
        obj["filterEffects"] = (
            FilterEffects.from_dict(obj["filterEffects"])
            if obj.get("filterEffects") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .ab_test import ABTest
from .ab_test_configuration import ABTestConfiguration
from .ab_test_response import ABTestResponse
from .ab_tests_variant import AbTestsVariant
from .ab_tests_variant_search_params import AbTestsVariantSearchParams
from .add_ab_tests_request import AddABTestsRequest
from .add_ab_tests_variant import AddABTestsVariant
from .currency import Currency
from .effect_metric import EffectMetric
from .empty_search import EmptySearch
from .empty_search_filter import EmptySearchFilter
from .error_base import ErrorBase
from .estimate_ab_test_request import EstimateABTestRequest
from .estimate_ab_test_response import EstimateABTestResponse
from .estimate_configuration import EstimateConfiguration
from .filter_effects import FilterEffects
from .list_ab_tests_response import ListABTestsResponse
from .minimum_detectable_effect import MinimumDetectableEffect
from .outliers import Outliers
from .outliers_filter import OutliersFilter
from .schedule_ab_test_response import ScheduleABTestResponse
from .schedule_ab_tests_request import ScheduleABTestsRequest
from .status import Status
from .variant import Variant

__all__ = (
    "ABTest",
    "ABTestConfiguration",
    "ABTestResponse",
    "AbTestsVariant",
    "AbTestsVariantSearchParams",
    "AddABTestsRequest",
    "AddABTestsVariant",
    "Currency",
    "EffectMetric",
    "EmptySearch",
    "EmptySearchFilter",
    "ErrorBase",
    "EstimateABTestRequest",
    "EstimateABTestResponse",
    "EstimateConfiguration",
    "FilterEffects",
    "ListABTestsResponse",
    "MinimumDetectableEffect",
    "Outliers",
    "OutliersFilter",
    "ScheduleABTestResponse",
    "ScheduleABTestsRequest",
    "Status",
    "Variant",
)



from asyncio import TimeoutError
from json import loads
from typing import List, Optional

from aiohttp import ClientSession, TCPConnector
from async_timeout import timeout

from algoliasearch.http.api_response import ApiResponse
from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.base_transporter import BaseTransporter
from algoliasearch.http.exceptions import (
    AlgoliaUnreachableHostException,
    RequestException,
)
from algoliasearch.http.hosts import Host
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.retry import RetryOutcome, RetryStrategy
from algoliasearch.http.verb import Verb


class Transporter(BaseTransporter):
    def __init__(self, config: BaseConfig) -> None:
        super().__init__(config)
        self._session: Optional[ClientSession] = None
        self._config = config
        self._retry_strategy = RetryStrategy()
        self._hosts: List[Host] = []

    async def close(self) -> None:
        if self._session is not None:
            _session = self._session
            self._session = None

            await _session.close()

    async def request(
        self,
        verb: Verb,
        path: str,
        request_options: RequestOptions,
        use_read_transporter: bool,
    ) -> ApiResponse:
        if self._session is None:
            self._session = ClientSession(
                connector=TCPConnector(use_dns_cache=False), trust_env=True
            )

        query_parameters = self.prepare(
            request_options, verb == Verb.GET or use_read_transporter
        )

        path = self.build_path(path, query_parameters)

        for host in self._retry_strategy.valid_hosts(self._hosts):
            url = self.build_url(host, path)
            proxy = self.get_proxy(url)

            try:
                async with timeout(self._timeout / 1000):
                    resp = await self._session.request(
                        method=verb,
                        url=url,
                        headers=request_options.headers,
                        data=request_options.data,
                        proxy=proxy,
                    )

                    _raw_data = await resp.text()
                    response = ApiResponse(
                        verb=verb,
                        path=path,
                        url=url,
                        host=host.url,
                        status_code=resp.status,
                        headers=resp.headers,  # pyright: ignore # insensitive dict is still a dict
                        data=_raw_data,
                        raw_data=_raw_data,
                        error_message=str(resp.reason),
                    )

            except TimeoutError as e:
                response = ApiResponse(
                    verb=verb,
                    path=path,
                    url=url,
                    host=host.url,
                    error_message=str(e),
                    is_timed_out_error=True,
                )

            decision = self._retry_strategy.decide(host, response)

            if decision == RetryOutcome.SUCCESS:
                return response
            elif decision == RetryOutcome.FAIL:
                content = response.error_message
                if response.data and "message" in response.data:
                    content = loads(response.data)["message"]

                raise RequestException(content, response.status_code)

        raise AlgoliaUnreachableHostException("Unreachable hosts")


class EchoTransporter(Transporter):
    def __init__(self, config: BaseConfig) -> None:
        super().__init__(config)
        self._config = config
        self._retry_strategy = RetryStrategy()

    async def request(
        self,
        verb: Verb,
        path: str,
        request_options: RequestOptions,
        use_read_transporter: bool,
    ) -> ApiResponse:
        self.prepare(request_options, verb == Verb.GET or use_read_transporter)

        return ApiResponse(
            verb=verb,
            path=path,
            status_code=200,
            host=self._retry_strategy.valid_hosts(self._hosts)[0].url,
            timeouts={
                "connect": request_options.timeouts["connect"],
                "response": self._timeout,
            },
            query_parameters=request_options.query_parameters,
            headers=dict(request_options.headers),
            data=request_options.data,
            raw_data=request_options.data,  # type: ignore
        )



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


_ALIASES = {
    "name": "name",
    "created_at": "createdAt",
    "updated_at": "updatedAt",
    "entries": "entries",
    "data_size": "dataSize",
    "file_size": "fileSize",
    "last_build_time_s": "lastBuildTimeS",
    "number_of_pending_tasks": "numberOfPendingTasks",
    "pending_task": "pendingTask",
    "primary": "primary",
    "replicas": "replicas",
    "virtual": "virtual",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class FetchedIndex(BaseModel):
    """
    FetchedIndex
    """

    name: str
    """ Index name. """
    created_at: str
    """ Index creation date. An empty string means that the index has no records. """
    updated_at: str
    """ Date and time when the object was updated, in RFC 3339 format. """
    entries: int
    """ Number of records contained in the index. """
    data_size: int
    """ Number of bytes of the index in minified format. """
    file_size: int
    """ Number of bytes of the index binary file. """
    last_build_time_s: int
    """ Last build time. """
    number_of_pending_tasks: int
    """ Number of pending indexing operations. This value is deprecated and should not be used. """
    pending_task: bool
    """ A boolean which says whether the index has pending tasks. This value is deprecated and should not be used. """
    primary: Optional[str] = None
    """ Only present if the index is a replica. Contains the name of the related primary index. """
    replicas: Optional[List[str]] = None
    """ Only present if the index is a primary index with replicas. Contains the names of all linked replicas. """
    virtual: Optional[bool] = None
    """ Only present if the index is a [virtual replica](https://www.algolia.com/doc/guides/managing-results/refine-results/sorting/how-to/sort-an-index-alphabetically/#virtual-replicas). """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of FetchedIndex from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of FetchedIndex from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return cls.model_validate(obj)



# coding: utf-8

import asyncio
import time
from typing import Awaitable, Callable, Optional, TypeVar

T = TypeVar("T")


class Timeout:
    def __call__(self, retry_count: int = 0) -> int:
        return retry_count

    def __init__(self) -> None:
        pass


class RetryTimeout(Timeout):
    def __call__(self, retry_count: int = 0) -> int:
        return int(min(retry_count * 0.2, 5))


async def create_iterable(
    func: Callable[[Optional[T]], Awaitable[T]],
    validate: Callable[[T], bool],
    aggregator: Optional[Callable[[T], None]],
    timeout: Callable[[], int] = Timeout(),
    error_validate: Optional[Callable[[T], bool]] = None,
    error_message: Optional[Callable[[T], str]] = None,
) -> T:
    """
    Helper: Iterates until the given `func` until `timeout` or `validate`.
    """

    async def retry(prev: Optional[T] = None) -> T:
        resp = await func(prev)

        if aggregator:
            aggregator(resp)

        if validate(resp):
            return resp

        if error_validate is not None and error_validate(resp):
            if error_message is None:
                raise Exception("An error occurred")
            raise Exception(error_message(resp))

        await asyncio.sleep(timeout())
        return await retry(resp)

    return await retry()


def create_iterable_sync(
    func: Callable[[Optional[T]], T],
    validate: Callable[[T], bool],
    aggregator: Optional[Callable[[T], None]],
    timeout: Callable[[], int] = Timeout(),
    error_validate: Optional[Callable[[T], bool]] = None,
    error_message: Optional[Callable[[T], str]] = None,
) -> T:
    """
    Helper: Iterates until the given `func` until `timeout` or `validate`.
    """

    def retry(prev: Optional[T] = None) -> T:
        resp = func(prev)

        if aggregator:
            aggregator(resp)

        if validate(resp):
            return resp

        if error_validate is not None and error_validate(resp):
            if error_message is None:
                raise Exception("An error occurred")
            raise Exception(error_message(resp))

        time.sleep(timeout())
        return retry(resp)

    return retry()



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


_ALIASES = {
    "duration_days": "durationDays",
    "sample_sizes": "sampleSizes",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class EstimateABTestResponse(BaseModel):
    """
    EstimateABTestResponse
    """

    duration_days: Optional[int] = None
    """ Estimated number of days needed to reach the sample sizes required for detecting the configured effect. This value is based on historical traffic. """
    sample_sizes: Optional[List[int]] = None
    """ Sample size estimates for each variant. The first element is the control variant. Each element is the estimated number of searches required to achieve the desired statistical significance.  """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of EstimateABTestResponse from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of EstimateABTestResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .error_base import ErrorBase
from .incident import Incident
from .incident_entry import IncidentEntry
from .incidents_response import IncidentsResponse
from .indexing_metric import IndexingMetric
from .indexing_time_response import IndexingTimeResponse
from .infrastructure_response import InfrastructureResponse
from .inventory_response import InventoryResponse
from .latency_metric import LatencyMetric
from .latency_response import LatencyResponse
from .metric import Metric
from .metrics import Metrics
from .period import Period
from .probes_metric import ProbesMetric
from .region import Region
from .server import Server
from .server_status import ServerStatus
from .status import Status
from .status_response import StatusResponse
from .time_entry import TimeEntry
from .type import Type

__all__ = (
    "ErrorBase",
    "Incident",
    "IncidentEntry",
    "IncidentsResponse",
    "IndexingMetric",
    "IndexingTimeResponse",
    "InfrastructureResponse",
    "InventoryResponse",
    "LatencyMetric",
    "LatencyResponse",
    "Metric",
    "Metrics",
    "Period",
    "ProbesMetric",
    "Region",
    "Server",
    "ServerStatus",
    "Status",
    "StatusResponse",
    "TimeEntry",
    "Type",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import dumps, loads
from sys import version_info
from typing import Any, Dict, List, Optional, Set, Union

from pydantic import BaseModel, Field, ValidationError, model_serializer

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


class OptionalWords(BaseModel):
    """
    Words that should be considered optional when found in the query.  By default, records must match all words in the search query to be included in the search results. Adding optional words can help to increase the number of search results by running an additional search query that doesn't include the optional words. For example, if the search query is \"action video\" and \"video\" is an optional word, the search engine runs two queries. One for \"action video\" and one for \"action\". Records that match all words are ranked higher.  For a search query with 4 or more words **and** all its words are optional, the number of matched words required for a record to be included in the search results increases for every 1,000 records:  - If `optionalWords` has less than 10 words, the required number of matched words increases by 1:   results 1 to 1,000 require 1 matched word, results 1,001 to 2000 need 2 matched words. - If `optionalWords` has 10 or more words, the number of required matched words increases by the number of optional words divided by 5 (rounded down).   For example, with 18 optional words: results 1 to 1,000 require 1 matched word, results 1,001 to 2000 need 4 matched words.  For more information, see [Optional words](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/empty-or-insufficient-results/#creating-a-list-of-optional-words).
    """

    oneof_schema_1_validator: Optional[str] = Field(default=None)

    oneof_schema_2_validator: Optional[List[str]] = Field(default=None)
    """ List of [optional words](https://www.algolia.com/doc/guides/managing-results/optimize-search-results/empty-or-insufficient-results/#creating-a-list-of-optional-words). """
    actual_instance: Union[List[str], str, None] = None
    one_of_schemas: Set[str] = {"List[str]", "str"}

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError(
                    "If a position argument is used, only 1 is allowed to set `actual_instance`"
                )
            if kwargs:
                raise ValueError(
                    "If a position argument is used, keyword arguments cannot be used."
                )
            super().__init__(actual_instance=args[0])  # pyright: ignore
        else:
            super().__init__(**kwargs)

    @model_serializer
    def unwrap_actual_instance(self) -> Union[List[str], str, Self, None]:
        """
        Unwraps the `actual_instance` when calling the `to_json` method.
        """
        return self.actual_instance if hasattr(self, "actual_instance") else self

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        """Create an instance of OptionalWords from a JSON string"""
        return cls.from_json(dumps(obj))

    @classmethod
    def from_json(cls, json_str: Optional[str]) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        if json_str is None:
            return instance

        error_messages = []

        try:
            instance.oneof_schema_1_validator = loads(json_str)
            instance.actual_instance = instance.oneof_schema_1_validator

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.oneof_schema_2_validator = loads(json_str)
            instance.actual_instance = instance.oneof_schema_2_validator

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        raise ValueError(
            "No match found when deserializing the JSON string into OptionalWords with oneOf schemas: List[str], str. Details: "
            + ", ".join(error_messages)
        )

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(
            self.actual_instance.to_json  # pyright: ignore
        ):
            return self.actual_instance.to_json()  # pyright: ignore
        else:
            return dumps(self.actual_instance)

    def to_dict(self) -> Optional[Union[Dict[str, Any], List[str], str]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(
            self.actual_instance.to_dict  # pyright: ignore
        ):
            return self.actual_instance.to_dict()  # pyright: ignore
        else:
            return self.actual_instance  # pyright: ignore



from json import loads
from sys import version_info

from requests import Request, Session, Timeout

if version_info >= (3, 11):
    from typing import List, Optional, Self
else:
    from typing_extensions import List, Self

from requests.adapters import HTTPAdapter
from urllib3.util import Retry

from algoliasearch.http.api_response import ApiResponse
from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.base_transporter import BaseTransporter
from algoliasearch.http.exceptions import (
    AlgoliaUnreachableHostException,
    RequestException,
)
from algoliasearch.http.hosts import Host
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.retry import RetryOutcome, RetryStrategy
from algoliasearch.http.verb import Verb


class TransporterSync(BaseTransporter):
    def __init__(self, config: BaseConfig) -> None:
        super().__init__(config)
        self._session: Optional[Session] = None
        self._config = config
        self._retry_strategy = RetryStrategy()
        self._hosts: List[Host] = []

    def __enter__(self) -> Self:
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        pass

    def close(self) -> None:
        if self._session is not None:
            _session = self._session
            self._session = None

            _session.close()

    def request(
        self,
        verb: Verb,
        path: str,
        request_options: RequestOptions,
        use_read_transporter: bool,
    ) -> ApiResponse:
        if self._session is None:
            self._session = Session()
            self._session.mount("https://", HTTPAdapter(max_retries=Retry(connect=0)))

        query_parameters = self.prepare(
            request_options, verb == Verb.GET or use_read_transporter
        )

        path = self.build_path(path, query_parameters)

        for host in self._retry_strategy.valid_hosts(self._hosts):
            url = self.build_url(host, path)
            proxies = self.get_proxies(url)

            req = Request(
                method=verb,
                url=url,
                headers=request_options.headers,
                data=request_options.data,
            ).prepare()

            try:
                resp = self._session.send(
                    req,
                    timeout=self._timeout / 1000,
                    proxies=proxies,
                )

                response = ApiResponse(
                    verb=verb,
                    path=path,
                    url=url,
                    host=host.url,
                    status_code=resp.status_code,
                    headers=resp.headers,  # type: ignore # insensitive dict is still a dict
                    data=resp.text,
                    raw_data=resp.text,
                    error_message=str(resp.reason),
                )
            except Timeout as e:
                response = ApiResponse(
                    verb=verb,
                    path=path,
                    url=url,
                    host=host.url,
                    error_message=str(e),
                    is_timed_out_error=True,
                )

            decision = self._retry_strategy.decide(host, response)

            if decision == RetryOutcome.SUCCESS:
                return response
            elif decision == RetryOutcome.FAIL:
                content = response.error_message
                if response.data and "message" in response.data:
                    content = loads(response.data)["message"]

                raise RequestException(content, response.status_code)

        raise AlgoliaUnreachableHostException("Unreachable hosts")


class EchoTransporterSync(TransporterSync):
    def __init__(self, config: BaseConfig) -> None:
        super().__init__(config)
        self._config = config
        self._retry_strategy = RetryStrategy()

    def request(
        self,
        verb: Verb,
        path: str,
        request_options: RequestOptions,
        use_read_transporter: bool,
    ) -> ApiResponse:
        self.prepare(request_options, verb == Verb.GET or use_read_transporter)

        return ApiResponse(
            verb=verb,
            path=path,
            status_code=200,
            host=self._retry_strategy.valid_hosts(self._hosts)[0].url,
            timeouts={
                "connect": request_options.timeouts["connect"],
                "response": self._timeout,
            },
            query_parameters=request_options.query_parameters,
            headers=dict(request_options.headers),
            data=request_options.data,
            raw_data=request_options.data,  # type: ignore
        )



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.recommend.models.fallback_params import FallbackParams
from algoliasearch.recommend.models.looking_similar_model import LookingSimilarModel
from algoliasearch.recommend.models.recommend_search_params import RecommendSearchParams

_ALIASES = {
    "index_name": "indexName",
    "threshold": "threshold",
    "max_recommendations": "maxRecommendations",
    "query_parameters": "queryParameters",
    "model": "model",
    "object_id": "objectID",
    "fallback_parameters": "fallbackParameters",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class LookingSimilarQuery(BaseModel):
    """
    LookingSimilarQuery
    """

    index_name: str
    """ Index name (case-sensitive). """
    threshold: float
    """ Minimum score a recommendation must have to be included in the response. """
    max_recommendations: Optional[int] = None
    """ Maximum number of recommendations to retrieve. By default, all recommendations are returned and no fallback request is made. Depending on the available recommendations and the other request parameters, the actual number of recommendations may be lower than this value.  """
    query_parameters: Optional[RecommendSearchParams] = None
    model: LookingSimilarModel
    object_id: str
    """ Unique record identifier. """
    fallback_parameters: Optional[FallbackParams] = None

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of LookingSimilarQuery from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of LookingSimilarQuery from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["queryParameters"] = (
            RecommendSearchParams.from_dict(obj["queryParameters"])
            if obj.get("queryParameters") is not None
            else None
        )
        obj["model"] = obj.get("model")
        obj["fallbackParameters"] = (
            FallbackParams.from_dict(obj["fallbackParameters"])
            if obj.get("fallbackParameters") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.around_radius import AroundRadius
from algoliasearch.search.models.facet_filters import FacetFilters
from algoliasearch.search.models.inside_bounding_box import InsideBoundingBox
from algoliasearch.search.models.numeric_filters import NumericFilters
from algoliasearch.search.models.tag_filters import TagFilters

_ALIASES = {
    "facet_filters": "facetFilters",
    "filters": "filters",
    "numeric_filters": "numericFilters",
    "tag_filters": "tagFilters",
    "around_lat_lng": "aroundLatLng",
    "around_radius": "aroundRadius",
    "inside_bounding_box": "insideBoundingBox",
    "inside_polygon": "insidePolygon",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class DeleteByParams(BaseModel):
    """
    DeleteByParams
    """

    facet_filters: Optional[FacetFilters] = None
    filters: Optional[str] = None
    """ Filter expression to only include items that match the filter criteria in the response.  You can use these filter expressions:  - **Numeric filters.** `<facet> <op> <number>`, where `<op>` is one of `<`, `<=`, `=`, `!=`, `>`, `>=`. - **Ranges.** `<facet>:<lower> TO <upper>` where `<lower>` and `<upper>` are the lower and upper limits of the range (inclusive). - **Facet filters.** `<facet>:<value>` where `<facet>` is a facet attribute (case-sensitive) and `<value>` a facet value. - **Tag filters.** `_tags:<value>` or just `<value>` (case-sensitive). - **Boolean filters.** `<facet>: true | false`.  You can combine filters with `AND`, `OR`, and `NOT` operators with the following restrictions:  - You can only combine filters of the same type with `OR`.   **Not supported:** `facet:value OR num > 3`. - You can't use `NOT` with combinations of filters.   **Not supported:** `NOT(facet:value OR facet:value)` - You can't combine conjunctions (`AND`) with `OR`.   **Not supported:** `facet:value OR (facet:value AND facet:value)`  Use quotes around your filters, if the facet attribute name or facet value has spaces, keywords (`OR`, `AND`, `NOT`), or quotes. If a facet attribute is an array, the filter matches if it matches at least one element of the array.  For more information, see [Filters](https://www.algolia.com/doc/guides/managing-results/refine-results/filtering/).  """
    numeric_filters: Optional[NumericFilters] = None
    tag_filters: Optional[TagFilters] = None
    around_lat_lng: Optional[str] = None
    """ Coordinates for the center of a circle, expressed as a comma-separated string of latitude and longitude.  Only records included within a circle around this central location are included in the results. The radius of the circle is determined by the `aroundRadius` and `minimumAroundRadius` settings. This parameter is ignored if you also specify `insidePolygon` or `insideBoundingBox`.  """
    around_radius: Optional[AroundRadius] = None
    inside_bounding_box: Optional[InsideBoundingBox] = None
    inside_polygon: Optional[List[List[float]]] = None
    """ Coordinates of a polygon in which to search.  Polygons are defined by 3 to 10,000 points. Each point is represented by its latitude and longitude. Provide multiple polygons as nested arrays. For more information, see [filtering inside polygons](https://www.algolia.com/doc/guides/managing-results/refine-results/geolocation/#filtering-inside-rectangular-or-polygonal-areas). This parameter is ignored if you also specify `insideBoundingBox`.  """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DeleteByParams from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DeleteByParams from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["facetFilters"] = (
            FacetFilters.from_dict(obj["facetFilters"])
            if obj.get("facetFilters") is not None
            else None
        )
        obj["numericFilters"] = (
            NumericFilters.from_dict(obj["numericFilters"])
            if obj.get("numericFilters") is not None
            else None
        )
        obj["tagFilters"] = (
            TagFilters.from_dict(obj["tagFilters"])
            if obj.get("tagFilters") is not None
            else None
        )
        obj["aroundRadius"] = (
            AroundRadius.from_dict(obj["aroundRadius"])
            if obj.get("aroundRadius") is not None
            else None
        )
        obj["insideBoundingBox"] = (
            InsideBoundingBox.from_dict(obj["insideBoundingBox"])
            if obj.get("insideBoundingBox") is not None
            else None
        )

        return cls.model_validate(obj)



from os import environ
from typing import Dict, Optional

from algoliasearch.http.hosts import HostsCollection
from algoliasearch.http.user_agent import UserAgent


class BaseConfig:
    def __init__(self, app_id: Optional[str] = None, api_key: Optional[str] = None):
        app_id = environ.get("ALGOLIA_APP_ID") if app_id is None else app_id

        assert app_id, "`app_id` is missing."

        api_key = environ.get("ALGOLIA_API_KEY") if api_key is None else api_key

        assert api_key, "`api_key` is missing."

        self.app_id = str(app_id)
        self.api_key = str(api_key)

        # In milliseconds
        self.read_timeout = 5000
        self.write_timeout = 30000
        self.connect_timeout = 2000

        self.wait_task_time_before_retry: Optional[int] = None
        self.headers: Dict[str, str] = {}
        self.proxies: Optional[Dict[str, str]] = None
        self.hosts: Optional[HostsCollection] = None

        self._user_agent: UserAgent = UserAgent()

    def set_client_api_key(self, api_key: str) -> None:
        """Sets a new API key to authenticate requests."""
        self.api_key = api_key
        self.headers["x-algolia-api-key"] = api_key

    def add_user_agent(self, segment: str, version: Optional[str] = None) -> None:
        """adds a segment to the default user agent, and update the headers sent with each requests as well"""
        self._user_agent = self._user_agent.add(segment, version)
        self.headers["user-agent"] = self._user_agent.get()



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.analytics.models.daily_no_click_rates import DailyNoClickRates

_ALIASES = {
    "rate": "rate",
    "count": "count",
    "no_click_count": "noClickCount",
    "dates": "dates",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class GetNoClickRateResponse(BaseModel):
    """
    GetNoClickRateResponse
    """

    rate: float
    """ No click rate: calculated as the number of tracked searches without clicks divided by the number of tracked searches.  """
    count: int
    """ Number of tracked searches. Tracked searches are search requests where the `clickAnalytics` parameter is true. """
    no_click_count: int
    """ Number of times this search was returned as a result without any click. """
    dates: List[DailyNoClickRates]
    """ Daily no click rates. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetNoClickRateResponse from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetNoClickRateResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["dates"] = (
            [DailyNoClickRates.from_dict(_item) for _item in obj["dates"]]
            if obj.get("dates") is not None
            else None
        )

        return cls.model_validate(obj)



from platform import python_version
from sys import version_info
from typing import Optional

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self

from algoliasearch import __version__


class UserAgent:
    def __init__(self) -> None:
        self.value = "Algolia for Python ({}); Python ({})".format(
            __version__, str(python_version())
        )

    def get(self) -> str:
        return self.value

    def add(self, segment: str, version: Optional[str] = None) -> Self:
        self.value += "; {}".format(segment)

        if version is not None:
            self.value += " ({})".format(version)

        return self



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import dumps
from sys import version_info
from typing import Any, Dict, Optional, Set, Union

from pydantic import BaseModel, Field, ValidationError, model_serializer

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.ingestion.models.transformation_code import TransformationCode
from algoliasearch.ingestion.models.transformation_no_code import TransformationNoCode


class TransformationInput(BaseModel):
    """
    The input for the transformation, which can be either code or a no-code configuration.
    """

    oneof_schema_1_validator: Optional[TransformationCode] = Field(default=None)

    oneof_schema_2_validator: Optional[TransformationNoCode] = Field(default=None)

    actual_instance: Union[TransformationCode, TransformationNoCode, None] = None
    one_of_schemas: Set[str] = {"TransformationCode", "TransformationNoCode"}

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError(
                    "If a position argument is used, only 1 is allowed to set `actual_instance`"
                )
            if kwargs:
                raise ValueError(
                    "If a position argument is used, keyword arguments cannot be used."
                )
            super().__init__(actual_instance=args[0])  # pyright: ignore
        else:
            super().__init__(**kwargs)

    @model_serializer
    def unwrap_actual_instance(
        self,
    ) -> Union[TransformationCode, TransformationNoCode, Self, None]:
        """
        Unwraps the `actual_instance` when calling the `to_json` method.
        """
        return self.actual_instance if hasattr(self, "actual_instance") else self

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        """Create an instance of TransformationInput from a JSON string"""
        return cls.from_json(dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []

        try:
            instance.actual_instance = TransformationCode.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = TransformationNoCode.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        raise ValueError(
            "No match found when deserializing the JSON string into TransformationInput with oneOf schemas: TransformationCode, TransformationNoCode. Details: "
            + ", ".join(error_messages)
        )

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(
            self.actual_instance.to_json  # pyright: ignore
        ):
            return self.actual_instance.to_json()  # pyright: ignore
        else:
            return dumps(self.actual_instance)

    def to_dict(
        self,
    ) -> Optional[Union[Dict[str, Any], TransformationCode, TransformationNoCode]]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(
            self.actual_instance.to_dict  # pyright: ignore
        ):
            return self.actual_instance.to_dict()  # pyright: ignore
        else:
            return self.actual_instance  # pyright: ignore



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .acl import Acl
from .action import Action
from .add_api_key_response import AddApiKeyResponse
from .advanced_syntax_features import AdvancedSyntaxFeatures
from .alternatives_as_exact import AlternativesAsExact
from .anchoring import Anchoring
from .api_key import ApiKey
from .around_precision import AroundPrecision
from .around_radius import AroundRadius
from .around_radius_all import AroundRadiusAll
from .assign_user_id_params import AssignUserIdParams
from .attribute_to_update import AttributeToUpdate
from .automatic_facet_filter import AutomaticFacetFilter
from .automatic_facet_filters import AutomaticFacetFilters
from .banner import Banner
from .banner_image import BannerImage
from .banner_image_url import BannerImageUrl
from .banner_link import BannerLink
from .batch_assign_user_ids_params import BatchAssignUserIdsParams
from .batch_dictionary_entries_params import BatchDictionaryEntriesParams
from .batch_dictionary_entries_request import BatchDictionaryEntriesRequest
from .batch_params import BatchParams
from .batch_request import BatchRequest
from .batch_response import BatchResponse
from .batch_write_params import BatchWriteParams
from .boolean_string import BooleanString
from .browse_params import BrowseParams
from .browse_params_object import BrowseParamsObject
from .browse_response import BrowseResponse
from .built_in_operation import BuiltInOperation
from .built_in_operation_type import BuiltInOperationType
from .built_in_operation_value import BuiltInOperationValue
from .condition import Condition
from .consequence import Consequence
from .consequence_hide import ConsequenceHide
from .consequence_params import ConsequenceParams
from .consequence_query import ConsequenceQuery
from .consequence_query_object import ConsequenceQueryObject
from .created_at_response import CreatedAtResponse
from .delete_api_key_response import DeleteApiKeyResponse
from .delete_by_params import DeleteByParams
from .delete_source_response import DeleteSourceResponse
from .deleted_at_response import DeletedAtResponse
from .dictionary_action import DictionaryAction
from .dictionary_entry import DictionaryEntry
from .dictionary_entry_state import DictionaryEntryState
from .dictionary_entry_type import DictionaryEntryType
from .dictionary_language import DictionaryLanguage
from .dictionary_settings_params import DictionarySettingsParams
from .dictionary_type import DictionaryType
from .distinct import Distinct
from .edit import Edit
from .edit_type import EditType
from .error_base import ErrorBase
from .event import Event
from .event_status import EventStatus
from .event_type import EventType
from .exact_on_single_word_query import ExactOnSingleWordQuery
from .exhaustive import Exhaustive
from .facet_filters import FacetFilters
from .facet_hits import FacetHits
from .facet_ordering import FacetOrdering
from .facet_stats import FacetStats
from .facets import Facets
from .fetched_index import FetchedIndex
from .get_api_key_response import GetApiKeyResponse
from .get_dictionary_settings_response import GetDictionarySettingsResponse
from .get_logs_response import GetLogsResponse
from .get_objects_params import GetObjectsParams
from .get_objects_request import GetObjectsRequest
from .get_objects_response import GetObjectsResponse
from .get_task_response import GetTaskResponse
from .get_top_user_ids_response import GetTopUserIdsResponse
from .has_pending_mappings_response import HasPendingMappingsResponse
from .highlight_result import HighlightResult
from .highlight_result_option import HighlightResultOption
from .hit import Hit
from .ignore_plurals import IgnorePlurals
from .index_settings import IndexSettings
from .inside_bounding_box import InsideBoundingBox
from .languages import Languages
from .list_api_keys_response import ListApiKeysResponse
from .list_clusters_response import ListClustersResponse
from .list_indices_response import ListIndicesResponse
from .list_user_ids_response import ListUserIdsResponse
from .log import Log
from .log_query import LogQuery
from .log_type import LogType
from .match_level import MatchLevel
from .matched_geo_location import MatchedGeoLocation
from .mode import Mode
from .multiple_batch_request import MultipleBatchRequest
from .multiple_batch_response import MultipleBatchResponse
from .numeric_filters import NumericFilters
from .operation_index_params import OperationIndexParams
from .operation_type import OperationType
from .optional_filters import OptionalFilters
from .optional_words import OptionalWords
from .personalization import Personalization
from .promote import Promote
from .promote_object_id import PromoteObjectID
from .promote_object_ids import PromoteObjectIDs
from .query_type import QueryType
from .range import Range
from .ranking_info import RankingInfo
from .re_ranking_apply_filter import ReRankingApplyFilter
from .redirect import Redirect
from .redirect_rule_index_data import RedirectRuleIndexData
from .redirect_rule_index_metadata import RedirectRuleIndexMetadata
from .redirect_url import RedirectURL
from .remove_stop_words import RemoveStopWords
from .remove_user_id_response import RemoveUserIdResponse
from .remove_words_if_no_results import RemoveWordsIfNoResults
from .rendering_content import RenderingContent
from .replace_all_objects_response import ReplaceAllObjectsResponse
from .replace_all_objects_with_transformation_response import (
    ReplaceAllObjectsWithTransformationResponse,
)
from .replace_source_response import ReplaceSourceResponse
from .rule import Rule
from .save_object_response import SaveObjectResponse
from .save_synonym_response import SaveSynonymResponse
from .scope_type import ScopeType
from .search_dictionary_entries_params import SearchDictionaryEntriesParams
from .search_dictionary_entries_response import SearchDictionaryEntriesResponse
from .search_for_facet_values_request import SearchForFacetValuesRequest
from .search_for_facet_values_response import SearchForFacetValuesResponse
from .search_for_facets import SearchForFacets
from .search_for_hits import SearchForHits
from .search_method_params import SearchMethodParams
from .search_params import SearchParams
from .search_params_object import SearchParamsObject
from .search_params_string import SearchParamsString
from .search_query import SearchQuery
from .search_response import SearchResponse
from .search_responses import SearchResponses
from .search_result import SearchResult
from .search_rules_params import SearchRulesParams
from .search_rules_response import SearchRulesResponse
from .search_strategy import SearchStrategy
from .search_synonyms_params import SearchSynonymsParams
from .search_synonyms_response import SearchSynonymsResponse
from .search_type_default import SearchTypeDefault
from .search_type_facet import SearchTypeFacet
from .search_user_ids_params import SearchUserIdsParams
from .search_user_ids_response import SearchUserIdsResponse
from .secured_api_key_restrictions import SecuredApiKeyRestrictions
from .semantic_search import SemanticSearch
from .settings_response import SettingsResponse
from .snippet_result import SnippetResult
from .snippet_result_option import SnippetResultOption
from .sort_remaining_by import SortRemainingBy
from .source import Source
from .standard_entries import StandardEntries
from .supported_language import SupportedLanguage
from .synonym_hit import SynonymHit
from .synonym_type import SynonymType
from .tag_filters import TagFilters
from .task_status import TaskStatus
from .time_range import TimeRange
from .typo_tolerance import TypoTolerance
from .typo_tolerance_enum import TypoToleranceEnum
from .update_api_key_response import UpdateApiKeyResponse
from .updated_at_response import UpdatedAtResponse
from .updated_at_with_object_id_response import UpdatedAtWithObjectIdResponse
from .user_highlight_result import UserHighlightResult
from .user_hit import UserHit
from .user_id import UserId
from .value import Value
from .watch_response import WatchResponse
from .widgets import Widgets

__all__ = (
    "ReplaceAllObjectsResponse",
    "ReplaceAllObjectsWithTransformationResponse",
    "SecuredApiKeyRestrictions",
    "Acl",
    "Action",
    "AddApiKeyResponse",
    "AdvancedSyntaxFeatures",
    "AlternativesAsExact",
    "Anchoring",
    "ApiKey",
    "AroundPrecision",
    "AroundRadius",
    "AroundRadiusAll",
    "AssignUserIdParams",
    "AttributeToUpdate",
    "AutomaticFacetFilter",
    "AutomaticFacetFilters",
    "Banner",
    "BannerImage",
    "BannerImageUrl",
    "BannerLink",
    "BatchAssignUserIdsParams",
    "BatchDictionaryEntriesParams",
    "BatchDictionaryEntriesRequest",
    "BatchParams",
    "BatchRequest",
    "BatchResponse",
    "BatchWriteParams",
    "BooleanString",
    "BrowseParams",
    "BrowseParamsObject",
    "BrowseResponse",
    "BuiltInOperation",
    "BuiltInOperationType",
    "BuiltInOperationValue",
    "Condition",
    "Consequence",
    "ConsequenceHide",
    "ConsequenceParams",
    "ConsequenceQuery",
    "ConsequenceQueryObject",
    "CreatedAtResponse",
    "DeleteApiKeyResponse",
    "DeleteByParams",
    "DeleteSourceResponse",
    "DeletedAtResponse",
    "DictionaryAction",
    "DictionaryEntry",
    "DictionaryEntryState",
    "DictionaryEntryType",
    "DictionaryLanguage",
    "DictionarySettingsParams",
    "DictionaryType",
    "Distinct",
    "Edit",
    "EditType",
    "ErrorBase",
    "Event",
    "EventStatus",
    "EventType",
    "ExactOnSingleWordQuery",
    "Exhaustive",
    "FacetFilters",
    "FacetHits",
    "FacetOrdering",
    "FacetStats",
    "Facets",
    "FetchedIndex",
    "GetApiKeyResponse",
    "GetDictionarySettingsResponse",
    "GetLogsResponse",
    "GetObjectsParams",
    "GetObjectsRequest",
    "GetObjectsResponse",
    "GetTaskResponse",
    "GetTopUserIdsResponse",
    "HasPendingMappingsResponse",
    "HighlightResult",
    "HighlightResultOption",
    "Hit",
    "IgnorePlurals",
    "IndexSettings",
    "InsideBoundingBox",
    "Languages",
    "ListApiKeysResponse",
    "ListClustersResponse",
    "ListIndicesResponse",
    "ListUserIdsResponse",
    "Log",
    "LogQuery",
    "LogType",
    "MatchLevel",
    "MatchedGeoLocation",
    "Mode",
    "MultipleBatchRequest",
    "MultipleBatchResponse",
    "NumericFilters",
    "OperationIndexParams",
    "OperationType",
    "OptionalFilters",
    "OptionalWords",
    "Personalization",
    "Promote",
    "PromoteObjectID",
    "PromoteObjectIDs",
    "QueryType",
    "Range",
    "RankingInfo",
    "ReRankingApplyFilter",
    "Redirect",
    "RedirectRuleIndexData",
    "RedirectRuleIndexMetadata",
    "RedirectURL",
    "RemoveStopWords",
    "RemoveUserIdResponse",
    "RemoveWordsIfNoResults",
    "RenderingContent",
    "ReplaceSourceResponse",
    "Rule",
    "SaveObjectResponse",
    "SaveSynonymResponse",
    "ScopeType",
    "SearchDictionaryEntriesParams",
    "SearchDictionaryEntriesResponse",
    "SearchForFacetValuesRequest",
    "SearchForFacetValuesResponse",
    "SearchForFacets",
    "SearchForHits",
    "SearchMethodParams",
    "SearchParams",
    "SearchParamsObject",
    "SearchParamsString",
    "SearchQuery",
    "SearchResponse",
    "SearchResponses",
    "SearchResult",
    "SearchRulesParams",
    "SearchRulesResponse",
    "SearchStrategy",
    "SearchSynonymsParams",
    "SearchSynonymsResponse",
    "SearchTypeDefault",
    "SearchTypeFacet",
    "SearchUserIdsParams",
    "SearchUserIdsResponse",
    "SemanticSearch",
    "SettingsResponse",
    "SnippetResult",
    "SnippetResultOption",
    "SortRemainingBy",
    "Source",
    "StandardEntries",
    "SupportedLanguage",
    "SynonymHit",
    "SynonymType",
    "TagFilters",
    "TaskStatus",
    "TimeRange",
    "TypoTolerance",
    "TypoToleranceEnum",
    "UpdateApiKeyResponse",
    "UpdatedAtResponse",
    "UpdatedAtWithObjectIdResponse",
    "UserHighlightResult",
    "UserHit",
    "UserId",
    "Value",
    "WatchResponse",
    "Widgets",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from sys import version_info

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


class LogLevel(str, Enum):
    """
    Type of log entry.  - `SKIP`. A query is skipped because it doesn't match the conditions for successful inclusion. For example, when a query doesn't generate enough search results. - `INFO`. An informative log entry. - `ERROR`. The Query Suggestions process encountered an error.
    """

    """
    allowed enum values
    """
    SKIP = "SKIP"

    INFO = "INFO"

    ERROR = "ERROR"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of LogLevel from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from sys import version_info

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


class Acl(str, Enum):
    """
    Access control list permissions.
    """

    """
    allowed enum values
    """
    ADDOBJECT = "addObject"

    ANALYTICS = "analytics"

    BROWSE = "browse"

    DELETEOBJECT = "deleteObject"

    DELETEINDEX = "deleteIndex"

    EDITSETTINGS = "editSettings"

    INFERENCE = "inference"

    LISTINDEXES = "listIndexes"

    LOGS = "logs"

    PERSONALIZATION = "personalization"

    RECOMMENDATION = "recommendation"

    SEARCH = "search"

    SEEUNRETRIEVABLEATTRIBUTES = "seeUnretrievableAttributes"

    SETTINGS = "settings"

    USAGE = "usage"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Acl from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict, field_validator

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.anchoring import Anchoring

_ALIASES = {
    "pattern": "pattern",
    "anchoring": "anchoring",
    "alternatives": "alternatives",
    "context": "context",
    "filters": "filters",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Condition(BaseModel):
    """
    Condition
    """

    pattern: Optional[str] = None
    """ Query pattern that triggers the rule.  You can use either a literal string, or a special pattern `{facet:ATTRIBUTE}`, where `ATTRIBUTE` is a facet name. The rule is triggered if the query matches the literal string or a value of the specified facet. For example, with `pattern: {facet:genre}`, the rule is triggered when users search for a genre, such as \"comedy\".  """
    anchoring: Optional[Anchoring] = None
    alternatives: Optional[bool] = None
    """ Whether the pattern should match plurals, synonyms, and typos. """
    context: Optional[str] = None
    """ An additional restriction that only triggers the rule, when the search has the same value as `ruleContexts` parameter. For example, if `context: mobile`, the rule is only triggered when the search request has a matching `ruleContexts: mobile`. A rule context must only contain alphanumeric characters.  """
    filters: Optional[str] = None
    """ Filters that trigger the rule.  You can add filters using the syntax `facet:value` so that the rule is triggered, when the specific filter is selected. You can use `filters` on its own or combine it with the `pattern` parameter. You can't combine multiple filters with `OR` and you can't use numeric filters.  """

    @field_validator("context")
    def context_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"[A-Za-z0-9_-]+", value):
            raise ValueError(r"must validate the regular expression /[A-Za-z0-9_-]+/")
        return value

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Condition from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Condition from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["anchoring"] = obj.get("anchoring")

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.facet_ordering import FacetOrdering
from algoliasearch.search.models.redirect_url import RedirectURL
from algoliasearch.search.models.widgets import Widgets

_ALIASES = {
    "facet_ordering": "facetOrdering",
    "redirect": "redirect",
    "widgets": "widgets",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class RenderingContent(BaseModel):
    """
    Extra data that can be used in the search UI.  You can use this to control aspects of your search UI, such as the order of facet names and values without changing your frontend code.
    """

    facet_ordering: Optional[FacetOrdering] = None
    redirect: Optional[RedirectURL] = None
    widgets: Optional[Widgets] = None

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RenderingContent from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RenderingContent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["facetOrdering"] = (
            FacetOrdering.from_dict(obj["facetOrdering"])
            if obj.get("facetOrdering") is not None
            else None
        )
        obj["redirect"] = (
            RedirectURL.from_dict(obj["redirect"])
            if obj.get("redirect") is not None
            else None
        )
        obj["widgets"] = (
            Widgets.from_dict(obj["widgets"])
            if obj.get("widgets") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import dumps
from sys import version_info
from typing import Any, Optional, Union
from urllib.parse import quote

from pydantic import Field, StrictStr
from typing_extensions import Annotated

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self

from algoliasearch.composition.config import CompositionConfig
from algoliasearch.composition.models import (
    RequestBody,
    SearchForFacetValuesRequest,
    SearchForFacetValuesResponse,
    SearchResponse,
)
from algoliasearch.http.api_response import ApiResponse
from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.request_options import RequestOptions
from algoliasearch.http.serializer import body_serializer
from algoliasearch.http.transporter import Transporter
from algoliasearch.http.transporter_sync import TransporterSync
from algoliasearch.http.verb import Verb


class CompositionClient:
    """The Algolia 'CompositionClient' class.

    Args:
    app_id (str): The Algolia application ID to retrieve information from.
    api_key (str): The Algolia api key bound to the given `app_id`.


    Returns:
    The initialized API client.

    Example:
    _client = CompositionClient("YOUR_ALGOLIA_APP_ID", "YOUR_ALGOLIA_API_KEY")
    _client_with_named_args = CompositionClient(app_id="YOUR_ALGOLIA_APP_ID", api_key="YOUR_ALGOLIA_API_KEY")

    See `CompositionClient.create_with_config` for advanced configuration.
    """

    _transporter: Transporter
    _config: BaseConfig
    _request_options: RequestOptions

    def __init__(
        self,
        app_id: Optional[str] = None,
        api_key: Optional[str] = None,
        transporter: Optional[Transporter] = None,
        config: Optional[CompositionConfig] = None,
    ) -> None:
        if transporter is not None and config is None:
            config = CompositionConfig(
                transporter.config.app_id, transporter.config.api_key
            )
        elif config is None:
            config = CompositionConfig(app_id, api_key)

        config.set_default_hosts()

        self._config = config
        self._request_options = RequestOptions(config)

        if transporter is None:
            transporter = Transporter(config)
        self._transporter = transporter

    @classmethod
    def create_with_config(
        cls, config: CompositionConfig, transporter: Optional[Transporter] = None
    ) -> CompositionClient:
        """Allows creating a client with a customized `CompositionConfig` and `Transporter`. If `transporter` is not provided, the default one will be initialized from the given `config`.

        Args:
        config (CompositionConfig): The config of the API client.
        transporter (Transporter): The HTTP transporter, see `http/transporter.py` for implementation details.

        Returns:
        The initialized API client.

        Example:
        _client_with_custom_config = CompositionClient.create_with_config(config=CompositionConfig(...))
        _client_with_custom_config_and_transporter = CompositionClient.create_with_config(config=CompositionConfig(...), transporter=Transporter(...))
        """
        if transporter is None:
            transporter = Transporter(config)

        client = CompositionClient(
            app_id=config.app_id,
            api_key=config.api_key,
            transporter=transporter,
            config=config,
        )

        return client

    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(self, exc_type, exc_value, traceback) -> None:
        """Closes the underlying `transporter` of the API client."""
        await self.close()

    async def close(self) -> None:
        """Closes the underlying `transporter` of the API client."""
        return await self._transporter.close()

    async def set_client_api_key(self, api_key: str) -> None:
        """Sets a new API key to authenticate requests."""
        self._transporter.config.set_client_api_key(api_key)

    async def add_user_agent(self, segment: str, version: Optional[str] = None) -> None:
        """adds a segment to the default user agent, and update the headers sent with each requests as well"""
        self._transporter.config.add_user_agent(segment, version)

    async def search_with_http_info(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        request_body: Union[RequestBody, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Runs a query on a single composition and returns matching results.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param request_body: (required)
        :type request_body: RequestBody
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if composition_id is None:
            raise ValueError(
                "Parameter `composition_id` is required when calling `search`."
            )

        if request_body is None:
            raise ValueError(
                "Parameter `request_body` is required when calling `search`."
            )

        _data = {}
        if request_body is not None:
            _data = request_body

        return await self._transporter.request(
            verb=Verb.POST,
            path="/1/compositions/{compositionID}/run".replace(
                "{compositionID}", quote(str(composition_id), safe="")
            ),
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=True,
        )

    async def search(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        request_body: Union[RequestBody, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> SearchResponse:
        """
        Runs a query on a single composition and returns matching results.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param request_body: (required)
        :type request_body: RequestBody
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'SearchResponse' result object.
        """
        resp = await self.search_with_http_info(
            composition_id, request_body, request_options
        )
        return resp.deserialize(SearchResponse, resp.raw_data)

    async def search_for_facet_values_with_http_info(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        facet_name: Annotated[
            StrictStr,
            Field(
                description="Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier. "
            ),
        ],
        search_for_facet_values_request: Union[
            Optional[SearchForFacetValuesRequest], dict[str, Any]
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Searches for values of a specified facet attribute on the composition's main source's index.  - By default, facet values are sorted by decreasing count.   You can adjust this with the `sortFacetValueBy` parameter. - Searching for facet values doesn't work if you have **more than 65 searchable facets and searchable attributes combined**.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param facet_name: Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier.  (required)
        :type facet_name: str
        :param search_for_facet_values_request:
        :type search_for_facet_values_request: SearchForFacetValuesRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if composition_id is None:
            raise ValueError(
                "Parameter `composition_id` is required when calling `search_for_facet_values`."
            )

        if facet_name is None:
            raise ValueError(
                "Parameter `facet_name` is required when calling `search_for_facet_values`."
            )

        _data = {}
        if search_for_facet_values_request is not None:
            _data = search_for_facet_values_request

        return await self._transporter.request(
            verb=Verb.POST,
            path="/1/compositions/{compositionID}/facets/{facetName}/query".replace(
                "{compositionID}", quote(str(composition_id), safe="")
            ).replace("{facetName}", quote(str(facet_name), safe="")),
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=True,
        )

    async def search_for_facet_values(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        facet_name: Annotated[
            StrictStr,
            Field(
                description="Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier. "
            ),
        ],
        search_for_facet_values_request: Union[
            Optional[SearchForFacetValuesRequest], dict[str, Any]
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> SearchForFacetValuesResponse:
        """
        Searches for values of a specified facet attribute on the composition's main source's index.  - By default, facet values are sorted by decreasing count.   You can adjust this with the `sortFacetValueBy` parameter. - Searching for facet values doesn't work if you have **more than 65 searchable facets and searchable attributes combined**.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param facet_name: Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier.  (required)
        :type facet_name: str
        :param search_for_facet_values_request:
        :type search_for_facet_values_request: SearchForFacetValuesRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'SearchForFacetValuesResponse' result object.
        """
        resp = await self.search_for_facet_values_with_http_info(
            composition_id, facet_name, search_for_facet_values_request, request_options
        )
        return resp.deserialize(SearchForFacetValuesResponse, resp.raw_data)


class CompositionClientSync:
    """The Algolia 'CompositionClientSync' class.

    Args:
    app_id (str): The Algolia application ID to retrieve information from.
    api_key (str): The Algolia api key bound to the given `app_id`.


    Returns:
    The initialized API client.

    Example:
    _client = CompositionClientSync("YOUR_ALGOLIA_APP_ID", "YOUR_ALGOLIA_API_KEY")
    _client_with_named_args = CompositionClientSync(app_id="YOUR_ALGOLIA_APP_ID", api_key="YOUR_ALGOLIA_API_KEY")

    See `CompositionClientSync.create_with_config` for advanced configuration.
    """

    _transporter: TransporterSync
    _config: BaseConfig
    _request_options: RequestOptions

    def __init__(
        self,
        app_id: Optional[str] = None,
        api_key: Optional[str] = None,
        transporter: Optional[TransporterSync] = None,
        config: Optional[CompositionConfig] = None,
    ) -> None:
        if transporter is not None and config is None:
            config = CompositionConfig(
                transporter.config.app_id, transporter.config.api_key
            )
        elif config is None:
            config = CompositionConfig(app_id, api_key)

        config.set_default_hosts()

        self._config = config
        self._request_options = RequestOptions(config)

        if transporter is None:
            transporter = TransporterSync(config)
        self._transporter = transporter

    @classmethod
    def create_with_config(
        cls, config: CompositionConfig, transporter: Optional[TransporterSync] = None
    ) -> CompositionClientSync:
        """Allows creating a client with a customized `CompositionConfig` and `TransporterSync`. If `transporter` is not provided, the default one will be initialized from the given `config`.

        Args:
        config (CompositionConfig): The config of the API client.
        transporter (TransporterSync): The HTTP transporter, see `http/transporter.py` for implementation details.

        Returns:
        The initialized API client.

        Example:
        _client_with_custom_config = CompositionClientSync.create_with_config(config=CompositionConfig(...))
        _client_with_custom_config_and_transporter = CompositionClientSync.create_with_config(config=CompositionConfig(...), transporter=TransporterSync(...))
        """
        if transporter is None:
            transporter = TransporterSync(config)

        client = CompositionClientSync(
            app_id=config.app_id,
            api_key=config.api_key,
            transporter=transporter,
            config=config,
        )

        return client

    def __enter__(self) -> Self:
        return self

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        """Closes the underlying `transporter` of the API client."""
        self.close()

    def close(self) -> None:
        return self._transporter.close()

    def set_client_api_key(self, api_key: str) -> None:
        """Sets a new API key to authenticate requests."""
        self._transporter.config.set_client_api_key(api_key)

    def add_user_agent(self, segment: str, version: Optional[str] = None) -> None:
        """adds a segment to the default user agent, and update the headers sent with each requests as well"""
        self._transporter.config.add_user_agent(segment, version)

    def search_with_http_info(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        request_body: Union[RequestBody, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Runs a query on a single composition and returns matching results.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param request_body: (required)
        :type request_body: RequestBody
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if composition_id is None:
            raise ValueError(
                "Parameter `composition_id` is required when calling `search`."
            )

        if request_body is None:
            raise ValueError(
                "Parameter `request_body` is required when calling `search`."
            )

        _data = {}
        if request_body is not None:
            _data = request_body

        return self._transporter.request(
            verb=Verb.POST,
            path="/1/compositions/{compositionID}/run".replace(
                "{compositionID}", quote(str(composition_id), safe="")
            ),
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=True,
        )

    def search(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        request_body: Union[RequestBody, dict[str, Any]],
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> SearchResponse:
        """
        Runs a query on a single composition and returns matching results.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param request_body: (required)
        :type request_body: RequestBody
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'SearchResponse' result object.
        """
        resp = self.search_with_http_info(composition_id, request_body, request_options)
        return resp.deserialize(SearchResponse, resp.raw_data)

    def search_for_facet_values_with_http_info(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        facet_name: Annotated[
            StrictStr,
            Field(
                description="Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier. "
            ),
        ],
        search_for_facet_values_request: Union[
            Optional[SearchForFacetValuesRequest], dict[str, Any]
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> ApiResponse[str]:
        """
        Searches for values of a specified facet attribute on the composition's main source's index.  - By default, facet values are sorted by decreasing count.   You can adjust this with the `sortFacetValueBy` parameter. - Searching for facet values doesn't work if you have **more than 65 searchable facets and searchable attributes combined**.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param facet_name: Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier.  (required)
        :type facet_name: str
        :param search_for_facet_values_request:
        :type search_for_facet_values_request: SearchForFacetValuesRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the raw algoliasearch 'APIResponse' object.
        """

        if composition_id is None:
            raise ValueError(
                "Parameter `composition_id` is required when calling `search_for_facet_values`."
            )

        if facet_name is None:
            raise ValueError(
                "Parameter `facet_name` is required when calling `search_for_facet_values`."
            )

        _data = {}
        if search_for_facet_values_request is not None:
            _data = search_for_facet_values_request

        return self._transporter.request(
            verb=Verb.POST,
            path="/1/compositions/{compositionID}/facets/{facetName}/query".replace(
                "{compositionID}", quote(str(composition_id), safe="")
            ).replace("{facetName}", quote(str(facet_name), safe="")),
            request_options=self._request_options.merge(
                data=dumps(body_serializer(_data)),
                user_request_options=request_options,
            ),
            use_read_transporter=True,
        )

    def search_for_facet_values(
        self,
        composition_id: Annotated[
            StrictStr, Field(description="Unique Composition ObjectID.")
        ],
        facet_name: Annotated[
            StrictStr,
            Field(
                description="Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier. "
            ),
        ],
        search_for_facet_values_request: Union[
            Optional[SearchForFacetValuesRequest], dict[str, Any]
        ] = None,
        request_options: Optional[Union[dict, RequestOptions]] = None,
    ) -> SearchForFacetValuesResponse:
        """
        Searches for values of a specified facet attribute on the composition's main source's index.  - By default, facet values are sorted by decreasing count.   You can adjust this with the `sortFacetValueBy` parameter. - Searching for facet values doesn't work if you have **more than 65 searchable facets and searchable attributes combined**.

        Required API Key ACLs:
          - search

        :param composition_id: Unique Composition ObjectID. (required)
        :type composition_id: str
        :param facet_name: Facet attribute in which to search for values.  This attribute must be included in the `attributesForFaceting` index setting with the `searchable()` modifier.  (required)
        :type facet_name: str
        :param search_for_facet_values_request:
        :type search_for_facet_values_request: SearchForFacetValuesRequest
        :param request_options: The request options to send along with the query, they will be merged with the transporter base parameters (headers, query params, timeouts, etc.). (optional)
        :return: Returns the deserialized response in a 'SearchForFacetValuesResponse' result object.
        """
        resp = self.search_for_facet_values_with_http_info(
            composition_id, facet_name, search_for_facet_values_request, request_options
        )
        return resp.deserialize(SearchForFacetValuesResponse, resp.raw_data)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict, field_validator

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.insights.models.discount import Discount
from algoliasearch.insights.models.price import Price

_ALIASES = {
    "query_id": "queryID",
    "price": "price",
    "quantity": "quantity",
    "discount": "discount",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class ObjectDataAfterSearch(BaseModel):
    """
    ObjectDataAfterSearch
    """

    query_id: Optional[str] = None
    """ Unique identifier for a search query, used to track purchase events with multiple records that originate from different searches. """
    price: Optional[Price] = None
    quantity: Optional[int] = None
    """ Quantity of a product that has been purchased or added to the cart. The total purchase value is the sum of `quantity` multiplied with the `price` for each purchased item.  """
    discount: Optional[Discount] = None

    @field_validator("query_id")
    def query_id_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"[0-9a-f]{32}", value):
            raise ValueError(r"must validate the regular expression /[0-9a-f]{32}/")
        return value

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ObjectDataAfterSearch from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ObjectDataAfterSearch from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["price"] = (
            Price.from_dict(obj["price"]) if obj.get("price") is not None else None
        )
        obj["discount"] = (
            Discount.from_dict(obj["discount"])
            if obj.get("discount") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .around_precision import AroundPrecision
from .around_radius import AroundRadius
from .around_radius_all import AroundRadiusAll
from .banner import Banner
from .banner_image import BannerImage
from .banner_image_url import BannerImageUrl
from .banner_link import BannerLink
from .composition_id_ranking_info import CompositionIdRankingInfo
from .composition_run_applied_rules import CompositionRunAppliedRules
from .composition_run_search_response import CompositionRunSearchResponse
from .compositions_search_response import CompositionsSearchResponse
from .error_base import ErrorBase
from .exhaustive import Exhaustive
from .facet_filters import FacetFilters
from .facet_hits import FacetHits
from .facet_ordering import FacetOrdering
from .facet_stats import FacetStats
from .facets import Facets
from .highlight_result import HighlightResult
from .highlight_result_option import HighlightResultOption
from .hit import Hit
from .hit_ranking_info import HitRankingInfo
from .inside_bounding_box import InsideBoundingBox
from .match_level import MatchLevel
from .matched_geo_location import MatchedGeoLocation
from .numeric_filters import NumericFilters
from .optional_filters import OptionalFilters
from .params import Params
from .personalization import Personalization
from .range import Range
from .redirect import Redirect
from .redirect_rule_index_data import RedirectRuleIndexData
from .redirect_rule_index_metadata import RedirectRuleIndexMetadata
from .redirect_url import RedirectURL
from .rendering_content import RenderingContent
from .request_body import RequestBody
from .results_composition_info_response import ResultsCompositionInfoResponse
from .results_injected_item_applied_rules_info_response import (
    ResultsInjectedItemAppliedRulesInfoResponse,
)
from .results_injected_item_info_response import ResultsInjectedItemInfoResponse
from .search_for_facet_values_params import SearchForFacetValuesParams
from .search_for_facet_values_request import SearchForFacetValuesRequest
from .search_for_facet_values_response import SearchForFacetValuesResponse
from .search_for_facet_values_results import SearchForFacetValuesResults
from .search_response import SearchResponse
from .search_results_item import SearchResultsItem
from .snippet_result import SnippetResult
from .snippet_result_option import SnippetResultOption
from .sort_remaining_by import SortRemainingBy
from .supported_language import SupportedLanguage
from .value import Value
from .widgets import Widgets

__all__ = (
    "AroundPrecision",
    "AroundRadius",
    "AroundRadiusAll",
    "Banner",
    "BannerImage",
    "BannerImageUrl",
    "BannerLink",
    "CompositionIdRankingInfo",
    "CompositionRunAppliedRules",
    "CompositionRunSearchResponse",
    "CompositionsSearchResponse",
    "ErrorBase",
    "Exhaustive",
    "FacetFilters",
    "FacetHits",
    "FacetOrdering",
    "FacetStats",
    "Facets",
    "HighlightResult",
    "HighlightResultOption",
    "Hit",
    "HitRankingInfo",
    "InsideBoundingBox",
    "MatchLevel",
    "MatchedGeoLocation",
    "NumericFilters",
    "OptionalFilters",
    "Params",
    "Personalization",
    "Range",
    "Redirect",
    "RedirectRuleIndexData",
    "RedirectRuleIndexMetadata",
    "RedirectURL",
    "RenderingContent",
    "RequestBody",
    "ResultsCompositionInfoResponse",
    "ResultsInjectedItemAppliedRulesInfoResponse",
    "ResultsInjectedItemInfoResponse",
    "SearchForFacetValuesParams",
    "SearchForFacetValuesRequest",
    "SearchForFacetValuesResponse",
    "SearchForFacetValuesResults",
    "SearchResponse",
    "SearchResultsItem",
    "SnippetResult",
    "SnippetResultOption",
    "SortRemainingBy",
    "SupportedLanguage",
    "Value",
    "Widgets",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .advanced_syntax_features import AdvancedSyntaxFeatures
from .alternatives_as_exact import AlternativesAsExact
from .around_precision import AroundPrecision
from .around_radius import AroundRadius
from .around_radius_all import AroundRadiusAll
from .auto_facet_filter import AutoFacetFilter
from .banner import Banner
from .banner_image import BannerImage
from .banner_image_url import BannerImageUrl
from .banner_link import BannerLink
from .boolean_string import BooleanString
from .bought_together_query import BoughtTogetherQuery
from .condition import Condition
from .consequence import Consequence
from .deleted_at_response import DeletedAtResponse
from .distinct import Distinct
from .error_base import ErrorBase
from .exact_on_single_word_query import ExactOnSingleWordQuery
from .exhaustive import Exhaustive
from .facet_filters import FacetFilters
from .facet_ordering import FacetOrdering
from .facet_stats import FacetStats
from .fallback_params import FallbackParams
from .fbt_model import FbtModel
from .get_recommend_task_response import GetRecommendTaskResponse
from .get_recommendations_params import GetRecommendationsParams
from .get_recommendations_response import GetRecommendationsResponse
from .hide_consequence_object import HideConsequenceObject
from .highlight_result import HighlightResult
from .highlight_result_option import HighlightResultOption
from .ignore_plurals import IgnorePlurals
from .index_settings_facets import IndexSettingsFacets
from .inside_bounding_box import InsideBoundingBox
from .looking_similar_model import LookingSimilarModel
from .looking_similar_query import LookingSimilarQuery
from .match_level import MatchLevel
from .matched_geo_location import MatchedGeoLocation
from .numeric_filters import NumericFilters
from .optional_filters import OptionalFilters
from .optional_words import OptionalWords
from .params_consequence import ParamsConsequence
from .personalization import Personalization
from .promote_consequence_object import PromoteConsequenceObject
from .query_type import QueryType
from .range import Range
from .ranking_info import RankingInfo
from .re_ranking_apply_filter import ReRankingApplyFilter
from .recommend_hit import RecommendHit
from .recommend_models import RecommendModels
from .recommend_rule import RecommendRule
from .recommend_search_params import RecommendSearchParams
from .recommend_updated_at_response import RecommendUpdatedAtResponse
from .recommendations_hit import RecommendationsHit
from .recommendations_request import RecommendationsRequest
from .recommendations_results import RecommendationsResults
from .redirect import Redirect
from .redirect_rule_index_data import RedirectRuleIndexData
from .redirect_rule_index_metadata import RedirectRuleIndexMetadata
from .redirect_url import RedirectURL
from .related_model import RelatedModel
from .related_query import RelatedQuery
from .remove_stop_words import RemoveStopWords
from .remove_words_if_no_results import RemoveWordsIfNoResults
from .rendering_content import RenderingContent
from .rule_metadata import RuleMetadata
from .search_recommend_rules_params import SearchRecommendRulesParams
from .search_recommend_rules_response import SearchRecommendRulesResponse
from .snippet_result import SnippetResult
from .snippet_result_option import SnippetResultOption
from .sort_remaining_by import SortRemainingBy
from .supported_language import SupportedLanguage
from .tag_filters import TagFilters
from .task_status import TaskStatus
from .time_range import TimeRange
from .trending_facet_hit import TrendingFacetHit
from .trending_facets_model import TrendingFacetsModel
from .trending_facets_query import TrendingFacetsQuery
from .trending_items_model import TrendingItemsModel
from .trending_items_query import TrendingItemsQuery
from .typo_tolerance import TypoTolerance
from .typo_tolerance_enum import TypoToleranceEnum
from .value import Value
from .widgets import Widgets

__all__ = (
    "AdvancedSyntaxFeatures",
    "AlternativesAsExact",
    "AroundPrecision",
    "AroundRadius",
    "AroundRadiusAll",
    "AutoFacetFilter",
    "Banner",
    "BannerImage",
    "BannerImageUrl",
    "BannerLink",
    "BooleanString",
    "BoughtTogetherQuery",
    "Condition",
    "Consequence",
    "DeletedAtResponse",
    "Distinct",
    "ErrorBase",
    "ExactOnSingleWordQuery",
    "Exhaustive",
    "FacetFilters",
    "FacetOrdering",
    "FacetStats",
    "FallbackParams",
    "FbtModel",
    "GetRecommendTaskResponse",
    "GetRecommendationsParams",
    "GetRecommendationsResponse",
    "HideConsequenceObject",
    "HighlightResult",
    "HighlightResultOption",
    "IgnorePlurals",
    "IndexSettingsFacets",
    "InsideBoundingBox",
    "LookingSimilarModel",
    "LookingSimilarQuery",
    "MatchLevel",
    "MatchedGeoLocation",
    "NumericFilters",
    "OptionalFilters",
    "OptionalWords",
    "ParamsConsequence",
    "Personalization",
    "PromoteConsequenceObject",
    "QueryType",
    "Range",
    "RankingInfo",
    "ReRankingApplyFilter",
    "RecommendHit",
    "RecommendModels",
    "RecommendRule",
    "RecommendSearchParams",
    "RecommendUpdatedAtResponse",
    "RecommendationsHit",
    "RecommendationsRequest",
    "RecommendationsResults",
    "Redirect",
    "RedirectRuleIndexData",
    "RedirectRuleIndexMetadata",
    "RedirectURL",
    "RelatedModel",
    "RelatedQuery",
    "RemoveStopWords",
    "RemoveWordsIfNoResults",
    "RenderingContent",
    "RuleMetadata",
    "SearchRecommendRulesParams",
    "SearchRecommendRulesResponse",
    "SnippetResult",
    "SnippetResultOption",
    "SortRemainingBy",
    "SupportedLanguage",
    "TagFilters",
    "TaskStatus",
    "TimeRange",
    "TrendingFacetHit",
    "TrendingFacetsModel",
    "TrendingFacetsQuery",
    "TrendingItemsModel",
    "TrendingItemsQuery",
    "TypoTolerance",
    "TypoToleranceEnum",
    "Value",
    "Widgets",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.matched_geo_location import MatchedGeoLocation
from algoliasearch.search.models.personalization import Personalization

_ALIASES = {
    "filters": "filters",
    "first_matched_word": "firstMatchedWord",
    "geo_distance": "geoDistance",
    "geo_precision": "geoPrecision",
    "matched_geo_location": "matchedGeoLocation",
    "personalization": "personalization",
    "nb_exact_words": "nbExactWords",
    "nb_typos": "nbTypos",
    "promoted": "promoted",
    "proximity_distance": "proximityDistance",
    "user_score": "userScore",
    "words": "words",
    "promoted_by_re_ranking": "promotedByReRanking",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class RankingInfo(BaseModel):
    """
    Object with detailed information about the record's ranking.
    """

    filters: Optional[int] = None
    """ Whether a filter matched the query. """
    first_matched_word: int
    """ Position of the first matched word in the best matching attribute of the record. """
    geo_distance: int
    """ Distance between the geo location in the search query and the best matching geo location in the record, divided by the geo precision (in meters). """
    geo_precision: Optional[int] = None
    """ Precision used when computing the geo distance, in meters. """
    matched_geo_location: Optional[MatchedGeoLocation] = None
    personalization: Optional[Personalization] = None
    nb_exact_words: int
    """ Number of exactly matched words. """
    nb_typos: int
    """ Number of typos encountered when matching the record. """
    promoted: Optional[bool] = None
    """ Whether the record was promoted by a rule. """
    proximity_distance: Optional[int] = None
    """ Number of words between multiple matches in the query plus 1. For single word queries, `proximityDistance` is 0. """
    user_score: int
    """ Overall ranking of the record, expressed as a single integer. This attribute is internal. """
    words: Optional[int] = None
    """ Number of matched words. """
    promoted_by_re_ranking: Optional[bool] = None
    """ Whether the record is re-ranked. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of RankingInfo from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of RankingInfo from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["matchedGeoLocation"] = (
            MatchedGeoLocation.from_dict(obj["matchedGeoLocation"])
            if obj.get("matchedGeoLocation") is not None
            else None
        )
        obj["personalization"] = (
            Personalization.from_dict(obj["personalization"])
            if obj.get("personalization") is not None
            else None
        )

        return cls.model_validate(obj)



from os import environ
from typing import Optional

from algoliasearch import __version__
from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.hosts import CallType, Host, HostsCollection
from algoliasearch.http.user_agent import UserAgent


class SearchConfig(BaseConfig):
    def __init__(self, app_id: Optional[str], api_key: Optional[str]) -> None:
        super().__init__(app_id, api_key)

        # In milliseconds
        self.read_timeout = 5000
        self.write_timeout = 30000
        self.connect_timeout = 2000

        self._user_agent = UserAgent()
        self.add_user_agent("Search", __version__)

        if app_id is None or not app_id:
            raise ValueError("`app_id` is missing.")

        if api_key is None or not api_key:
            raise ValueError("`api_key` is missing.")

        self.headers = {
            "x-algolia-application-id": app_id,
            "x-algolia-api-key": api_key,
            "user-agent": self._user_agent.get(),
            "content-type": "application/json",
        }

        http_proxy = environ.get("HTTP_PROXY")
        https_proxy = environ.get("HTTPS_PROXY")

        self.proxies = {}

        if http_proxy is not None:
            self.proxies["http"] = http_proxy
        if https_proxy is not None:
            self.proxies["https"] = https_proxy

        self.region = None

    def set_transformation_region(self, region: str = ""):
        "This method is required to be called with the appropriate region of your Algolia application if you wish to leverage the *_with_transformation methods."
        self.region = region

    def set_default_hosts(self):
        if self.hosts is not None:
            return

        self.hosts = HostsCollection(
            [
                Host(
                    url="{}-dsn.algolia.net".format(self.app_id),
                    priority=10,
                    accept=CallType.READ,
                ),
                Host(
                    url="{}.algolia.net".format(self.app_id),
                    priority=10,
                    accept=CallType.WRITE,
                ),
                Host("{}-1.algolianet.com".format(self.app_id)),
                Host("{}-2.algolianet.com".format(self.app_id)),
                Host("{}-3.algolianet.com".format(self.app_id)),
            ],
            reorder_hosts=True,
        )



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .action import Action
from .action_type import ActionType
from .auth_algolia import AuthAlgolia
from .auth_algolia_insights import AuthAlgoliaInsights
from .auth_algolia_insights_partial import AuthAlgoliaInsightsPartial
from .auth_algolia_partial import AuthAlgoliaPartial
from .auth_api_key import AuthAPIKey
from .auth_api_key_partial import AuthAPIKeyPartial
from .auth_basic import AuthBasic
from .auth_basic_partial import AuthBasicPartial
from .auth_google_service_account import AuthGoogleServiceAccount
from .auth_google_service_account_partial import AuthGoogleServiceAccountPartial
from .auth_input import AuthInput
from .auth_input_partial import AuthInputPartial
from .auth_o_auth import AuthOAuth
from .auth_o_auth_partial import AuthOAuthPartial
from .authentication import Authentication
from .authentication_create import AuthenticationCreate
from .authentication_create_response import AuthenticationCreateResponse
from .authentication_search import AuthenticationSearch
from .authentication_sort_keys import AuthenticationSortKeys
from .authentication_type import AuthenticationType
from .authentication_update import AuthenticationUpdate
from .authentication_update_response import AuthenticationUpdateResponse
from .big_commerce_channel import BigCommerceChannel
from .big_commerce_metafield import BigCommerceMetafield
from .big_query_data_type import BigQueryDataType
from .commercetools_custom_fields import CommercetoolsCustomFields
from .delete_response import DeleteResponse
from .destination import Destination
from .destination_create import DestinationCreate
from .destination_create_response import DestinationCreateResponse
from .destination_input import DestinationInput
from .destination_search import DestinationSearch
from .destination_sort_keys import DestinationSortKeys
from .destination_type import DestinationType
from .destination_update import DestinationUpdate
from .destination_update_response import DestinationUpdateResponse
from .docker_streams import DockerStreams
from .docker_streams_input import DockerStreamsInput
from .docker_streams_sync_mode import DockerStreamsSyncMode
from .email_notifications import EmailNotifications
from .entity_type import EntityType
from .error_base import ErrorBase
from .event import Event
from .event_sort_keys import EventSortKeys
from .event_status import EventStatus
from .event_type import EventType
from .list_authentications_response import ListAuthenticationsResponse
from .list_destinations_response import ListDestinationsResponse
from .list_events_response import ListEventsResponse
from .list_sources_response import ListSourcesResponse
from .list_tasks_response import ListTasksResponse
from .list_tasks_response_v1 import ListTasksResponseV1
from .list_transformations_response import ListTransformationsResponse
from .mapping_field_directive import MappingFieldDirective
from .mapping_format_schema import MappingFormatSchema
from .mapping_input import MappingInput
from .mapping_kit_action import MappingKitAction
from .mapping_type_csv import MappingTypeCSV
from .method_type import MethodType
from .notifications import Notifications
from .on_demand_trigger import OnDemandTrigger
from .on_demand_trigger_input import OnDemandTriggerInput
from .on_demand_trigger_type import OnDemandTriggerType
from .order_keys import OrderKeys
from .pagination import Pagination
from .platform import Platform
from .platform_none import PlatformNone
from .platform_with_none import PlatformWithNone
from .policies import Policies
from .push_task_payload import PushTaskPayload
from .push_task_records import PushTaskRecords
from .record_type import RecordType
from .run import Run
from .run_list_response import RunListResponse
from .run_outcome import RunOutcome
from .run_progress import RunProgress
from .run_reason_code import RunReasonCode
from .run_response import RunResponse
from .run_sort_keys import RunSortKeys
from .run_source_payload import RunSourcePayload
from .run_source_response import RunSourceResponse
from .run_status import RunStatus
from .run_type import RunType
from .schedule_trigger import ScheduleTrigger
from .schedule_trigger_input import ScheduleTriggerInput
from .schedule_trigger_type import ScheduleTriggerType
from .shopify_input import ShopifyInput
from .shopify_market import ShopifyMarket
from .shopify_metafield import ShopifyMetafield
from .source import Source
from .source_big_commerce import SourceBigCommerce
from .source_big_query import SourceBigQuery
from .source_commercetools import SourceCommercetools
from .source_create import SourceCreate
from .source_create_response import SourceCreateResponse
from .source_csv import SourceCSV
from .source_docker import SourceDocker
from .source_ga4_big_query_export import SourceGA4BigQueryExport
from .source_input import SourceInput
from .source_json import SourceJSON
from .source_search import SourceSearch
from .source_shopify import SourceShopify
from .source_sort_keys import SourceSortKeys
from .source_type import SourceType
from .source_update import SourceUpdate
from .source_update_commercetools import SourceUpdateCommercetools
from .source_update_docker import SourceUpdateDocker
from .source_update_input import SourceUpdateInput
from .source_update_response import SourceUpdateResponse
from .source_update_shopify import SourceUpdateShopify
from .streaming_input import StreamingInput
from .streaming_trigger import StreamingTrigger
from .streaming_trigger_type import StreamingTriggerType
from .subscription_trigger import SubscriptionTrigger
from .subscription_trigger_type import SubscriptionTriggerType
from .task import Task
from .task_create import TaskCreate
from .task_create_response import TaskCreateResponse
from .task_create_trigger import TaskCreateTrigger
from .task_create_v1 import TaskCreateV1
from .task_input import TaskInput
from .task_search import TaskSearch
from .task_sort_keys import TaskSortKeys
from .task_update import TaskUpdate
from .task_update_response import TaskUpdateResponse
from .task_update_v1 import TaskUpdateV1
from .task_v1 import TaskV1
from .transformation import Transformation
from .transformation_code import TransformationCode
from .transformation_create import TransformationCreate
from .transformation_create_response import TransformationCreateResponse
from .transformation_error import TransformationError
from .transformation_input import TransformationInput
from .transformation_no_code import TransformationNoCode
from .transformation_search import TransformationSearch
from .transformation_sort_keys import TransformationSortKeys
from .transformation_try import TransformationTry
from .transformation_try_response import TransformationTryResponse
from .transformation_type import TransformationType
from .transformation_update_response import TransformationUpdateResponse
from .trigger import Trigger
from .trigger_type import TriggerType
from .trigger_update_input import TriggerUpdateInput
from .watch_response import WatchResponse
from .window import Window

__all__ = (
    "Action",
    "ActionType",
    "AuthAPIKey",
    "AuthAPIKeyPartial",
    "AuthAlgolia",
    "AuthAlgoliaInsights",
    "AuthAlgoliaInsightsPartial",
    "AuthAlgoliaPartial",
    "AuthBasic",
    "AuthBasicPartial",
    "AuthGoogleServiceAccount",
    "AuthGoogleServiceAccountPartial",
    "AuthInput",
    "AuthInputPartial",
    "AuthOAuth",
    "AuthOAuthPartial",
    "Authentication",
    "AuthenticationCreate",
    "AuthenticationCreateResponse",
    "AuthenticationSearch",
    "AuthenticationSortKeys",
    "AuthenticationType",
    "AuthenticationUpdate",
    "AuthenticationUpdateResponse",
    "BigCommerceChannel",
    "BigCommerceMetafield",
    "BigQueryDataType",
    "CommercetoolsCustomFields",
    "DeleteResponse",
    "Destination",
    "DestinationCreate",
    "DestinationCreateResponse",
    "DestinationInput",
    "DestinationSearch",
    "DestinationSortKeys",
    "DestinationType",
    "DestinationUpdate",
    "DestinationUpdateResponse",
    "DockerStreams",
    "DockerStreamsInput",
    "DockerStreamsSyncMode",
    "EmailNotifications",
    "EntityType",
    "ErrorBase",
    "Event",
    "EventSortKeys",
    "EventStatus",
    "EventType",
    "ListAuthenticationsResponse",
    "ListDestinationsResponse",
    "ListEventsResponse",
    "ListSourcesResponse",
    "ListTasksResponse",
    "ListTasksResponseV1",
    "ListTransformationsResponse",
    "MappingFieldDirective",
    "MappingFormatSchema",
    "MappingInput",
    "MappingKitAction",
    "MappingTypeCSV",
    "MethodType",
    "Notifications",
    "OnDemandTrigger",
    "OnDemandTriggerInput",
    "OnDemandTriggerType",
    "OrderKeys",
    "Pagination",
    "Platform",
    "PlatformNone",
    "PlatformWithNone",
    "Policies",
    "PushTaskPayload",
    "PushTaskRecords",
    "RecordType",
    "Run",
    "RunListResponse",
    "RunOutcome",
    "RunProgress",
    "RunReasonCode",
    "RunResponse",
    "RunSortKeys",
    "RunSourcePayload",
    "RunSourceResponse",
    "RunStatus",
    "RunType",
    "ScheduleTrigger",
    "ScheduleTriggerInput",
    "ScheduleTriggerType",
    "ShopifyInput",
    "ShopifyMarket",
    "ShopifyMetafield",
    "Source",
    "SourceBigCommerce",
    "SourceBigQuery",
    "SourceCSV",
    "SourceCommercetools",
    "SourceCreate",
    "SourceCreateResponse",
    "SourceDocker",
    "SourceGA4BigQueryExport",
    "SourceInput",
    "SourceJSON",
    "SourceSearch",
    "SourceShopify",
    "SourceSortKeys",
    "SourceType",
    "SourceUpdate",
    "SourceUpdateCommercetools",
    "SourceUpdateDocker",
    "SourceUpdateInput",
    "SourceUpdateResponse",
    "SourceUpdateShopify",
    "StreamingInput",
    "StreamingTrigger",
    "StreamingTriggerType",
    "SubscriptionTrigger",
    "SubscriptionTriggerType",
    "Task",
    "TaskCreate",
    "TaskCreateResponse",
    "TaskCreateTrigger",
    "TaskCreateV1",
    "TaskInput",
    "TaskSearch",
    "TaskSortKeys",
    "TaskUpdate",
    "TaskUpdateResponse",
    "TaskUpdateV1",
    "TaskV1",
    "Transformation",
    "TransformationCode",
    "TransformationCreate",
    "TransformationCreateResponse",
    "TransformationError",
    "TransformationInput",
    "TransformationNoCode",
    "TransformationSearch",
    "TransformationSortKeys",
    "TransformationTry",
    "TransformationTryResponse",
    "TransformationType",
    "TransformationUpdateResponse",
    "Trigger",
    "TriggerType",
    "TriggerUpdateInput",
    "WatchResponse",
    "Window",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .base_response import BaseResponse
from .config_status import ConfigStatus
from .configuration import Configuration
from .configuration_response import ConfigurationResponse
from .configuration_with_index import ConfigurationWithIndex
from .error_base import ErrorBase
from .facet import Facet
from .languages import Languages
from .log_file import LogFile
from .log_level import LogLevel
from .source_index import SourceIndex

__all__ = (
    "BaseResponse",
    "ConfigStatus",
    "Configuration",
    "ConfigurationResponse",
    "ConfigurationWithIndex",
    "ErrorBase",
    "Facet",
    "Languages",
    "LogFile",
    "LogLevel",
    "SourceIndex",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.condition import Condition
from algoliasearch.search.models.consequence import Consequence
from algoliasearch.search.models.time_range import TimeRange

_ALIASES = {
    "object_id": "objectID",
    "conditions": "conditions",
    "consequence": "consequence",
    "description": "description",
    "enabled": "enabled",
    "validity": "validity",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Rule(BaseModel):
    """
    Rule object.
    """

    object_id: str
    """ Unique identifier of a rule object. """
    conditions: Optional[List[Condition]] = None
    """ Conditions that trigger a rule.  Some consequences require specific conditions or don't require any condition. For more information, see [Conditions](https://www.algolia.com/doc/guides/managing-results/rules/rules-overview/#conditions).  """
    consequence: Consequence
    description: Optional[str] = None
    """ Description of the rule's purpose to help you distinguish between different rules. """
    enabled: Optional[bool] = None
    """ Whether the rule is active. """
    validity: Optional[List[TimeRange]] = None
    """ Time periods when the rule is active. """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Rule from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Rule from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["conditions"] = (
            [Condition.from_dict(_item) for _item in obj["conditions"]]
            if obj.get("conditions") is not None
            else None
        )
        obj["consequence"] = (
            Consequence.from_dict(obj["consequence"])
            if obj.get("consequence") is not None
            else None
        )
        obj["validity"] = (
            [TimeRange.from_dict(_item) for _item in obj["validity"]]
            if obj.get("validity") is not None
            else None
        )

        return cls.model_validate(obj)



from os import environ
from typing import Optional

from algoliasearch import __version__
from algoliasearch.http.base_config import BaseConfig
from algoliasearch.http.hosts import Host, HostsCollection
from algoliasearch.http.user_agent import UserAgent


class InsightsConfig(BaseConfig):
    def __init__(
        self,
        app_id: Optional[str],
        api_key: Optional[str],
        region: Optional[str] = None,
    ) -> None:
        super().__init__(app_id, api_key)

        # In milliseconds
        self.read_timeout = 5000
        self.write_timeout = 30000
        self.connect_timeout = 2000

        self._user_agent = UserAgent()
        self.add_user_agent("Insights", __version__)

        if app_id is None or not app_id:
            raise ValueError("`app_id` is missing.")

        if api_key is None or not api_key:
            raise ValueError("`api_key` is missing.")

        self.headers = {
            "x-algolia-application-id": app_id,
            "x-algolia-api-key": api_key,
            "user-agent": self._user_agent.get(),
            "content-type": "application/json",
        }

        http_proxy = environ.get("HTTP_PROXY")
        https_proxy = environ.get("HTTPS_PROXY")

        self.proxies = {}

        if http_proxy is not None:
            self.proxies["http"] = http_proxy
        if https_proxy is not None:
            self.proxies["https"] = https_proxy

        self.region = region

    def set_default_hosts(self):
        if self.hosts is not None:
            return

        _regions = ["de", "us"]

        if self.region is not None and self.region not in _regions:
            raise ValueError(
                f"`region` must be one of the following: {', '.join(_regions)}"
            )

        self.hosts = HostsCollection(
            [
                Host(
                    "insights.algolia.io"
                    if self.region is None
                    else "insights.{region}.algolia.io".replace(
                        "{region}", self.region or ""
                    )
                )
            ]
        )



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from enum import Enum
from json import loads
from sys import version_info

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


class Region(str, Enum):
    """
    Region where the cluster is located.
    """

    """
    allowed enum values
    """
    AU = "au"

    BR = "br"

    CA = "ca"

    DE = "de"

    EU = "eu"

    HK = "hk"

    IN = "in"

    JP = "jp"

    SG = "sg"

    UAE = "uae"

    UK = "uk"

    USC = "usc"

    USE = "use"

    USW = "usw"

    ZA = "za"

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of Region from a JSON string"""
        return cls(loads(json_str))



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from re import match
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict, field_validator

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.insights.models.click_event import ClickEvent

_ALIASES = {
    "event_name": "eventName",
    "event_type": "eventType",
    "index": "index",
    "filters": "filters",
    "user_token": "userToken",
    "authenticated_user_token": "authenticatedUserToken",
    "timestamp": "timestamp",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class ClickedFilters(BaseModel):
    """
    Use this event to track when users click facet filters in your user interface.
    """

    event_name: str
    """ Event name, up to 64 ASCII characters.  Consider naming events consistently—for example, by adopting Segment's [object-action](https://segment.com/academy/collecting-data/naming-conventions-for-clean-data/#the-object-action-framework) framework.  """
    event_type: ClickEvent
    index: str
    """ Index name (case-sensitive) to which the event's items belong. """
    filters: List[str]
    """ Applied facet filters.  Facet filters are `facet:value` pairs. Facet values must be URL-encoded, such as, `discount:10%25`.  """
    user_token: str
    """ Anonymous or pseudonymous user identifier.  Don't use personally identifiable information in user tokens. For more information, see [User token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/).  """
    authenticated_user_token: Optional[str] = None
    """ Identifier for authenticated users.  When the user signs in, you can get an identifier from your system and send it as `authenticatedUserToken`. This lets you keep using the `userToken` from before the user signed in, while providing a reliable way to identify users across sessions. Don't use personally identifiable information in user tokens. For more information, see [User token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/).  """
    timestamp: Optional[int] = None
    """ Timestamp of the event, measured in milliseconds since the Unix epoch. By default, the Insights API uses the time it receives an event as its timestamp.  """

    @field_validator("event_name")
    def event_name_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[\x20-\x7E]{1,64}", value):
            raise ValueError(
                r"must validate the regular expression /[\x20-\x7E]{1,64}/"
            )
        return value

    @field_validator("user_token")
    def user_token_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if not match(r"[a-zA-Z0-9_=\/+-]{1,129}", value):
            raise ValueError(
                r"must validate the regular expression /[a-zA-Z0-9_=\/+-]{1,129}/"
            )
        return value

    @field_validator("authenticated_user_token")
    def authenticated_user_token_validate_regular_expression(cls, value):
        """Validates the regular expression"""
        if value is None:
            return value

        if not match(r"[a-zA-Z0-9_=\/+-]{1,129}", value):
            raise ValueError(
                r"must validate the regular expression /[a-zA-Z0-9_=\/+-]{1,129}/"
            )
        return value

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ClickedFilters from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ClickedFilters from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["eventType"] = obj.get("eventType")

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.ingestion.models.destination_input import DestinationInput
from algoliasearch.ingestion.models.destination_type import DestinationType

_ALIASES = {
    "type": "type",
    "name": "name",
    "input": "input",
    "authentication_id": "authenticationID",
    "transformation_ids": "transformationIDs",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class DestinationCreate(BaseModel):
    """
    API request body for creating a new destination.
    """

    type: DestinationType
    name: str
    """ Descriptive name for the resource. """
    input: DestinationInput
    authentication_id: Optional[str] = None
    """ Universally unique identifier (UUID) of an authentication resource. """
    transformation_ids: Optional[List[str]] = None

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of DestinationCreate from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of DestinationCreate from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["type"] = obj.get("type")
        obj["input"] = (
            DestinationInput.from_dict(obj["input"])
            if obj.get("input") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.consequence_hide import ConsequenceHide
from algoliasearch.search.models.consequence_params import ConsequenceParams
from algoliasearch.search.models.promote import Promote

_ALIASES = {
    "params": "params",
    "promote": "promote",
    "filter_promotes": "filterPromotes",
    "hide": "hide",
    "user_data": "userData",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class Consequence(BaseModel):
    """
    Effect of the rule.  For more information, see [Consequences](https://www.algolia.com/doc/guides/managing-results/rules/rules-overview/#consequences).
    """

    params: Optional[ConsequenceParams] = None
    promote: Optional[List[Promote]] = None
    """ Records you want to pin to a specific position in the search results.  You can promote up to 300 records, either individually, or as groups of up to 100 records each.  """
    filter_promotes: Optional[bool] = None
    """ Whether promoted records must match an active filter for the consequence to be applied.  This ensures that user actions (filtering the search) are given a higher precendence. For example, if you promote a record with the `color: red` attribute, and the user filters the search for `color: blue`, the \"red\" record won't be shown.  """
    hide: Optional[List[ConsequenceHide]] = None
    """ Records you want to hide from the search results. """
    user_data: Optional[object] = None
    """ A JSON object with custom data that will be appended to the `userData` array in the response. This object isn't interpreted by the API and is limited to 1&nbsp;kB of minified JSON.  """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Consequence from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Consequence from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["params"] = (
            ConsequenceParams.from_dict(obj["params"])
            if obj.get("params") is not None
            else None
        )
        obj["promote"] = (
            [Promote.from_dict(_item) for _item in obj["promote"]]
            if obj.get("promote") is not None
            else None
        )
        obj["hide"] = (
            [ConsequenceHide.from_dict(_item) for _item in obj["hide"]]
            if obj.get("hide") is not None
            else None
        )

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

# The version of the algoliasearch package
__version__ = "4.21.0"


from .add_to_cart_event import AddToCartEvent
from .added_to_cart_object_ids import AddedToCartObjectIDs
from .added_to_cart_object_ids_after_search import AddedToCartObjectIDsAfterSearch
from .click_event import ClickEvent
from .clicked_filters import ClickedFilters
from .clicked_object_ids import ClickedObjectIDs
from .clicked_object_ids_after_search import ClickedObjectIDsAfterSearch
from .conversion_event import ConversionEvent
from .converted_filters import ConvertedFilters
from .converted_object_ids import ConvertedObjectIDs
from .converted_object_ids_after_search import ConvertedObjectIDsAfterSearch
from .discount import Discount
from .error_base import ErrorBase
from .events_items import EventsItems
from .events_response import EventsResponse
from .insights_events import InsightsEvents
from .object_data import ObjectData
from .object_data_after_search import ObjectDataAfterSearch
from .price import Price
from .purchase_event import PurchaseEvent
from .purchased_object_ids import PurchasedObjectIDs
from .purchased_object_ids_after_search import PurchasedObjectIDsAfterSearch
from .value import Value
from .view_event import ViewEvent
from .viewed_filters import ViewedFilters
from .viewed_object_ids import ViewedObjectIDs

__all__ = (
    "AddToCartEvent",
    "AddedToCartObjectIDs",
    "AddedToCartObjectIDsAfterSearch",
    "ClickEvent",
    "ClickedFilters",
    "ClickedObjectIDs",
    "ClickedObjectIDsAfterSearch",
    "ConversionEvent",
    "ConvertedFilters",
    "ConvertedObjectIDs",
    "ConvertedObjectIDsAfterSearch",
    "Discount",
    "ErrorBase",
    "EventsItems",
    "EventsResponse",
    "InsightsEvents",
    "ObjectData",
    "ObjectDataAfterSearch",
    "Price",
    "PurchaseEvent",
    "PurchasedObjectIDs",
    "PurchasedObjectIDsAfterSearch",
    "Value",
    "ViewEvent",
    "ViewedFilters",
    "ViewedObjectIDs",
)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import loads
from sys import version_info
from typing import Any, Dict, List, Optional

from pydantic import BaseModel, ConfigDict

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.search.models.acl import Acl

_ALIASES = {
    "value": "value",
    "created_at": "createdAt",
    "acl": "acl",
    "description": "description",
    "indexes": "indexes",
    "max_hits_per_query": "maxHitsPerQuery",
    "max_queries_per_ip_per_hour": "maxQueriesPerIPPerHour",
    "query_parameters": "queryParameters",
    "referers": "referers",
    "validity": "validity",
}


def _alias_generator(name: str) -> str:
    return _ALIASES.get(name, name)


class GetApiKeyResponse(BaseModel):
    """
    GetApiKeyResponse
    """

    value: str
    """ API key. """
    created_at: int
    """ Timestamp when the object was created, in milliseconds since the Unix epoch. """
    acl: List[Acl]
    """ Permissions that determine the type of API requests this key can make. The required ACL is listed in each endpoint's reference. For more information, see [access control list](https://www.algolia.com/doc/guides/security/api-keys/#access-control-list-acl).  """
    description: Optional[str] = None
    """ Description of an API key to help you identify this API key. """
    indexes: Optional[List[str]] = None
    """ Index names or patterns that this API key can access. By default, an API key can access all indices in the same application.  You can use leading and trailing wildcard characters (`*`):  - `dev_*` matches all indices starting with \"dev_\". - `*_dev` matches all indices ending with \"_dev\". - `*_products_*` matches all indices containing \"_products_\".  """
    max_hits_per_query: Optional[int] = None
    """ Maximum number of results this API key can retrieve in one query. By default, there's no limit.  """
    max_queries_per_ip_per_hour: Optional[int] = None
    """ Maximum number of API requests allowed per IP address or [user token](https://www.algolia.com/doc/guides/sending-events/concepts/usertoken/) per hour.  If this limit is reached, the API returns an error with status code `429`. By default, there's no limit.  """
    query_parameters: Optional[str] = None
    """ Query parameters to add when making API requests with this API key.  To restrict this API key to specific IP addresses, add the `restrictSources` parameter. You can only add a single source, but you can provide a range of IP addresses.  Creating an API key fails if the request is made from an IP address outside the restricted range.  """
    referers: Optional[List[str]] = None
    """ Allowed HTTP referrers for this API key.  By default, all referrers are allowed. You can use leading and trailing wildcard characters (`*`):  - `https://algolia.com/*` allows all referrers starting with \"https://algolia.com/\" - `*.algolia.com` allows all referrers ending with \".algolia.com\" - `*algolia.com*` allows all referrers in the domain \"algolia.com\".  Like all HTTP headers, referrers can be spoofed. Don't rely on them to secure your data. For more information, see [HTTP referrer restrictions](https://www.algolia.com/doc/guides/security/security-best-practices/#http-referrers-restrictions).  """
    validity: Optional[int] = None
    """ Duration (in seconds) after which the API key expires. By default, API keys don't expire.  """

    model_config = ConfigDict(
        strict=False,
        use_enum_values=True,
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
        alias_generator=_alias_generator,
        extra="allow",
    )

    def to_json(self) -> str:
        return self.model_dump_json(by_alias=True, exclude_unset=True)

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of GetApiKeyResponse from a JSON string"""
        return cls.from_dict(loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias."""
        return self.model_dump(
            by_alias=True,
            exclude_none=True,
            exclude_unset=True,
        )

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of GetApiKeyResponse from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        obj["acl"] = obj.get("acl")

        return cls.model_validate(obj)



# coding: utf-8

"""
Code generated by OpenAPI Generator (https://openapi-generator.tech), manual changes will be lost - read more on https://github.com/algolia/api-clients-automation. DO NOT EDIT.
"""

from __future__ import annotations

from json import dumps
from sys import version_info
from typing import Any, Dict, Optional, Set, Union

from pydantic import BaseModel, Field, ValidationError, model_serializer

if version_info >= (3, 11):
    from typing import Self
else:
    from typing_extensions import Self


from algoliasearch.insights.models.added_to_cart_object_ids import AddedToCartObjectIDs
from algoliasearch.insights.models.added_to_cart_object_ids_after_search import (
    AddedToCartObjectIDsAfterSearch,
)
from algoliasearch.insights.models.clicked_filters import ClickedFilters
from algoliasearch.insights.models.clicked_object_ids import ClickedObjectIDs
from algoliasearch.insights.models.clicked_object_ids_after_search import (
    ClickedObjectIDsAfterSearch,
)
from algoliasearch.insights.models.converted_filters import ConvertedFilters
from algoliasearch.insights.models.converted_object_ids import ConvertedObjectIDs
from algoliasearch.insights.models.converted_object_ids_after_search import (
    ConvertedObjectIDsAfterSearch,
)
from algoliasearch.insights.models.purchased_object_ids import PurchasedObjectIDs
from algoliasearch.insights.models.purchased_object_ids_after_search import (
    PurchasedObjectIDsAfterSearch,
)
from algoliasearch.insights.models.viewed_filters import ViewedFilters
from algoliasearch.insights.models.viewed_object_ids import ViewedObjectIDs


class EventsItems(BaseModel):
    """
    EventsItems
    """

    oneof_schema_1_validator: Optional[ClickedObjectIDsAfterSearch] = Field(
        default=None
    )

    oneof_schema_2_validator: Optional[AddedToCartObjectIDsAfterSearch] = Field(
        default=None
    )

    oneof_schema_3_validator: Optional[PurchasedObjectIDsAfterSearch] = Field(
        default=None
    )

    oneof_schema_4_validator: Optional[ConvertedObjectIDsAfterSearch] = Field(
        default=None
    )

    oneof_schema_5_validator: Optional[ClickedObjectIDs] = Field(default=None)

    oneof_schema_6_validator: Optional[PurchasedObjectIDs] = Field(default=None)

    oneof_schema_7_validator: Optional[AddedToCartObjectIDs] = Field(default=None)

    oneof_schema_8_validator: Optional[ConvertedObjectIDs] = Field(default=None)

    oneof_schema_9_validator: Optional[ClickedFilters] = Field(default=None)

    oneof_schema_10_validator: Optional[ConvertedFilters] = Field(default=None)

    oneof_schema_11_validator: Optional[ViewedObjectIDs] = Field(default=None)

    oneof_schema_12_validator: Optional[ViewedFilters] = Field(default=None)

    actual_instance: Union[
        AddedToCartObjectIDs,
        AddedToCartObjectIDsAfterSearch,
        ClickedFilters,
        ClickedObjectIDs,
        ClickedObjectIDsAfterSearch,
        ConvertedFilters,
        ConvertedObjectIDs,
        ConvertedObjectIDsAfterSearch,
        PurchasedObjectIDs,
        PurchasedObjectIDsAfterSearch,
        ViewedFilters,
        ViewedObjectIDs,
        None,
    ] = None
    one_of_schemas: Set[str] = {
        "AddedToCartObjectIDs",
        "AddedToCartObjectIDsAfterSearch",
        "ClickedFilters",
        "ClickedObjectIDs",
        "ClickedObjectIDsAfterSearch",
        "ConvertedFilters",
        "ConvertedObjectIDs",
        "ConvertedObjectIDsAfterSearch",
        "PurchasedObjectIDs",
        "PurchasedObjectIDsAfterSearch",
        "ViewedFilters",
        "ViewedObjectIDs",
    }

    def __init__(self, *args, **kwargs) -> None:
        if args:
            if len(args) > 1:
                raise ValueError(
                    "If a position argument is used, only 1 is allowed to set `actual_instance`"
                )
            if kwargs:
                raise ValueError(
                    "If a position argument is used, keyword arguments cannot be used."
                )
            super().__init__(actual_instance=args[0])  # pyright: ignore
        else:
            super().__init__(**kwargs)

    @model_serializer
    def unwrap_actual_instance(
        self,
    ) -> Union[
        AddedToCartObjectIDs,
        AddedToCartObjectIDsAfterSearch,
        ClickedFilters,
        ClickedObjectIDs,
        ClickedObjectIDsAfterSearch,
        ConvertedFilters,
        ConvertedObjectIDs,
        ConvertedObjectIDsAfterSearch,
        PurchasedObjectIDs,
        PurchasedObjectIDsAfterSearch,
        ViewedFilters,
        ViewedObjectIDs,
        Self,
        None,
    ]:
        """
        Unwraps the `actual_instance` when calling the `to_json` method.
        """
        return self.actual_instance if hasattr(self, "actual_instance") else self

    @classmethod
    def from_dict(cls, obj: Union[str, Dict[str, Any]]) -> Self:
        """Create an instance of EventsItems from a JSON string"""
        return cls.from_json(dumps(obj))

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Returns the object represented by the json string"""
        instance = cls.model_construct()
        error_messages = []

        try:
            instance.actual_instance = ClickedObjectIDsAfterSearch.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = AddedToCartObjectIDsAfterSearch.from_json(
                json_str
            )

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = PurchasedObjectIDsAfterSearch.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ConvertedObjectIDsAfterSearch.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ClickedObjectIDs.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = PurchasedObjectIDs.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = AddedToCartObjectIDs.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ConvertedObjectIDs.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ClickedFilters.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ConvertedFilters.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ViewedObjectIDs.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))
        try:
            instance.actual_instance = ViewedFilters.from_json(json_str)

            return instance
        except (ValidationError, ValueError) as e:
            error_messages.append(str(e))

        raise ValueError(
            "No match found when deserializing the JSON string into EventsItems with oneOf schemas: AddedToCartObjectIDs, AddedToCartObjectIDsAfterSearch, ClickedFilters, ClickedObjectIDs, ClickedObjectIDsAfterSearch, ConvertedFilters, ConvertedObjectIDs, ConvertedObjectIDsAfterSearch, PurchasedObjectIDs, PurchasedObjectIDsAfterSearch, ViewedFilters, ViewedObjectIDs. Details: "
            + ", ".join(error_messages)
        )

    def to_json(self) -> str:
        """Returns the JSON representation of the actual instance"""
        if self.actual_instance is None:
            return "null"

        if hasattr(self.actual_instance, "to_json") and callable(
            self.actual_instance.to_json  # pyright: ignore
        ):
            return self.actual_instance.to_json()  # pyright: ignore
        else:
            return dumps(self.actual_instance)

    def to_dict(
        self,
    ) -> Optional[
        Union[
            Dict[str, Any],
            AddedToCartObjectIDs,
            AddedToCartObjectIDsAfterSearch,
            ClickedFilters,
            ClickedObjectIDs,
            ClickedObjectIDsAfterSearch,
            ConvertedFilters,
            ConvertedObjectIDs,
            ConvertedObjectIDsAfterSearch,
            PurchasedObjectIDs,
            PurchasedObjectIDsAfterSearch,
            ViewedFilters,
            ViewedObjectIDs,
        ]
    ]:
        """Returns the dict representation of the actual instance"""
        if self.actual_instance is None:
            return None

        if hasattr(self.actual_instance, "to_dict") and callable(
            self.actual_instance.to_dict  # pyright: ignore
        ):
            return self.actual_instance.to_dict()  # pyright: ignore
        else:
            return self.actual_instance  # pyright: ignore
